-- GLOBALS: BWsavedtable, BWconfig

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    local _G = _G
    local unpack = unpack
    local math = math
    local pairs = pairs
    local tonumber = tonumber

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local InterfaceOptions_AddCategory = InterfaceOptions_AddCategory
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UIDropDownMenu_SetSelectedValue = UIDropDownMenu_SetSelectedValue
    local UIDROPDOWNMENU_MAXBUTTONS = UIDROPDOWNMENU_MAXBUTTONS
    local OpacitySliderFrame = OpacitySliderFrame
    local ColorPickerFrame = ColorPickerFrame
    local ReloadUI = ReloadUI

    local B, L, T = unpack(select(2, ...))
    local SML = LibStub('LibSharedMedia-3.0')

    -- key binding interface constants
    _G['BINDING_HEADER_BW'] = "Bank's Watch"
    _G['BINDING_NAME_BW_TOGGLE'] = L[10020]
    _G['BINDING_NAME_BW_TOGGLETRACKER'] = L[10056]
    _G['BINDING_NAME_BW_BANK'] = L[10022]
    _G['BINDING_NAME_BW_REAGENT'] = L[10023]
    _G['BINDING_NAME_BW_CHARACTER'] = L[10024]
    _G['BINDING_NAME_BW_VOID'] = L[10025]
    _G['BINDING_NAME_BW_GUILD'] = L[10026]
    _G["BINDING_NAME_CLICK BWartifact:LeftButtonDown"] = L[10021].." "..L[9010]

    B.C = {}

--[[//////////////////////////////////////////////////

    CONFIGS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    function B.C.Create()
        -- http://wowwiki.wikia.com/wiki/Using_the_Interface_Options_Addons_panel
        local addon = {}
        
        addon.panel = CreateFrame("Frame", "BWPanel", UIParent)
        addon.panel:SetScript("OnShow", function(self)  end)
        -- Register in the Interface Addon Options GUI
        -- Set the name for the Category for the Options Panel
        addon.panel.name = "Bank's Watch"
        addon.panel.title = addon.panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        addon.panel.title:SetFont(addon.panel.title:GetFont(), 16) -- Resize string
        addon.panel.title:SetText("Bank's Watch")
        addon.panel.title:SetPoint("TOPLEFT", 15, -15)

        addon.panel.support = addon.panel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
        addon.panel.support:SetPoint("TOPLEFT", addon.panel.title, "BOTTOMLEFT", 0, -10)
        addon.panel.support:SetText(L[10028])

        -- Add the panel to the Interface Options
        addon.panel.perconfig = B.C.perconfigcheckbox(addon.panel)
        addon.panel.importexport = B.C.importexport(addon.panel)
        addon.panel.iconsize = B.C.iconsizeslider(addon.panel)
        addon.panel.iconrow = B.C.iconrowslider(addon.panel)
        addon.panel.reload = B.C.reloaduibutton(addon.panel, L[10029])
        addon.panel.minimize = B.C.minimizecheckbox(addon.panel)
        addon.panel.openbank = B.C.openbankcheckbox(addon.panel)
        addon.panel.openplayer = B.C.openplayercheckbox(addon.panel)
        addon.panel.fontquantity = B.C.FontQuantityLabel.Create(addon.panel)
        addon.panel.fontoutline = B.C.FontOutlineLabel.Create(addon.panel)
        -- Create tabs (color picker and label)
        addon.panel.colorsection = B.C.colorselect(addon.panel)

        addon.panel.default = B.C.ConfigToDefault
        addon.panel.refresh = B.C.ConfigRefresh
        InterfaceOptions_AddCategory(addon.panel)

        B.C['panel'] = addon.panel
    end

    function B.C.ResetColor(r, g, b, a, colorTable, label, option, button)

        colorTable.r = r; colorTable.g = g; colorTable.b = b; colorTable.a = a

        T.CONFIG['color'..option..'label'] = nil
        T.CONFIG['color'..option] = nil
        _G['BW'..option..'coloroption']:SetBackdropColor(colorTable.r, colorTable.g, colorTable.b, colorTable.a)
        _G['BW'..option..'coloroption']:SetBackdropBorderColor(1, 1, 1, 0.3)
        _G['BW'..option..'coloroption'].editbox:SetText(label)
        button:SetText("|cffffffff"..label.."|r")
        button:SetBackdropColor(colorTable.r, colorTable.g,colorTable.b, colorTable.a)
    end

    local originalfont, originalsize, originaloutline
    function B.C.ResetButtons(self)
        local i, button
        local listFrame = _G['BWquantityfontdropDownMenu']
        for i = 1, listFrame.numButtons do
            button = _G[listFrame.."Button"..i];
            B.C.ResetButton(button)
        end
    end

    function B.C.ResetButton(self)
        local normalText

        normalText = _G[self:GetName().."NormalText"]
        normalText:SetFont(originalfont, originalsize, originaloutline)

        self:SetScript("OnHide", nil)
    end

    function B.C.ConfigToDefault()

        T.CONFIG['iconsize'] = nil
        T.CONFIG['iconrow'] = nil
        T.CONFIG['minimize'] = nil
        T.CONFIG['openbank'] = nil
        T.CONFIG['openplayer'] = nil
        T.CONFIG['FontQuantity'] = "default"
        T.CONFIG['FontQuantityOutline'] = "OUTLINE"
        T.FONTS['quantity'] = T.FONTS[1]
        T.FONTS['quantityoutline'] = T.CONFIG['FontQuantityOutline']

        B.C.Font_OnChanged(_G['BWquantityfontdropDownMenu'], T.CONFIG['FontQuantity'])
        UIDropDownMenu_SetSelectedValue(_G['BWquantityoutlinedropDownMenu'], T.CONFIG['FontQuantityOutline'])

        T.LABEL['bank'] = L[10001]
        B.C.ResetColor(0.32, 0.15, 0.64, 1, T.BANKTAB, T.LABEL['bank'], "bank", T.F.mainframe.bankbutton)

        T.LABEL['reagent'] = L[10002]
        B.C.ResetColor(0.34, 0.24, 0.49, 1, T.REAGENTTAB, T.LABEL['reagent'], "reagent", T.F.mainframe.reagentbutton)

        T.LABEL['player'] = L[10010]
        B.C.ResetColor(0.39, 0.24, 0.81, 1, T.CHARTAB, T.LABEL['player'], "character", T.F.mainframe.characterbutton)

        T.LABEL['void'] = L[10003]
        B.C.ResetColor(0.24, 0.24, 0.49, 1, T.VOIDTAB, T.LABEL['void'], "void", T.F.mainframe.voidbutton)

        T.LABEL['guild'] = L[10004]
        B.C.ResetColor(0.20, 0.17, 0.56, 1, T.GUILDTAB, T.LABEL['guild'], "guild", T.F.mainframe.guildbutton)

        T.TOOLTIP.r = T.ITEM_QUALITY_COLORS[6].r; T.TOOLTIP.g = T.ITEM_QUALITY_COLORS[6].g; T.TOOLTIP.b = T.ITEM_QUALITY_COLORS[6].b; T.TOOLTIP.a = 1;
        T.TOOLTIPAC.r = T.ITEM_QUALITY_COLORS[5].r; T.TOOLTIPAC.g = T.ITEM_QUALITY_COLORS[5].g; T.TOOLTIPAC.b = T.ITEM_QUALITY_COLORS[5].b; T.TOOLTIPAC.a = 1;
        T.THEME.r = 0.05; T.THEME.g = 0.05; T.THEME.b = 0.05; T.THEME.a = 0.5;
        T.TRANSPARENCY.r = 0; T.TRANSPARENCY.g = 0; T.TRANSPARENCY.b = 0; T.TRANSPARENCY.a = 0.5;
        T.TRANSPARENCY_SLOT.r = 0; T.TRANSPARENCY_SLOT.g = 0; T.TRANSPARENCY_SLOT.b = 0; T.TRANSPARENCY_SLOT.a = 0.2;
        T.BORDER_SLOT.r = 0; T.BORDER_SLOT.g = 0; T.BORDER_SLOT.b = 0; T.BORDER_SLOT.a = 1;

        T.CONFIG['colortooltip'] = nil
        T.CONFIG['colortooltipaccent'] = nil
        T.CONFIG['transparency'] = nil
        T.CONFIG['transparencyslot'] = nil
        T.CONFIG['borderslot'] = nil
        T.CONFIG['basecolor'] = nil

        B.AdjustTransparency()
        B.ResetFontButtons()
    end

    function B.C.ConfigRefresh()
        local r, g, b, a

        _G['BWperconfigcheckbox']:SetChecked(T.CONFIG['perconfig'])
        _G['BWiconsizeslider']:SetValue(T.CONFIG['iconsize'] or 32)
        _G['BWiconrowslider']:SetValue(T.CONFIG['iconrow'] or 12)
        _G['BWminimizecheckbox']:SetChecked(T.CONFIG['minimize'])
        _G['BWopenbankcheckbox']:SetChecked(T.CONFIG['openbank'])
        _G['BWopenplayercheckbox']:SetChecked(T.CONFIG['openplayer'])

        B.C.Font_OnChanged(_G['BWquantityfontdropDownMenu'], T.CONFIG['FontQuantity'] or "default")
        UIDropDownMenu_SetSelectedValue(_G['BWquantityoutlinedropDownMenu'], T.CONFIG['FontQuantityOutline'] or "OUTLINE")
        _G['BWquantityoutlinedropDownMenuText']:SetText(T.CONFIG['FontQuantityOutline'] or "OUTLINE") -- For some reason, it sets it as Custom.

        r, g, b, a = T.CONFIG['colorbank'] and unpack(T.CONFIG['colorbank']) or T.BANKTAB.r, T.BANKTAB.g, T.BANKTAB.b, T.BANKTAB.a
        _G['BWbankcoloroption']:SetBackdropColor(r, g, b, a)
        _G['BWbankcoloroption'].editbox:SetText(T.LABEL['bank'])

        r, g, b, a = T.CONFIG['colorreagent'] and unpack(T.CONFIG['colorreagent']) or T.REAGENTTAB.r, T.REAGENTTAB.g, T.REAGENTTAB.b, T.REAGENTTAB.a
        _G['BWreagentcoloroption']:SetBackdropColor(r, g, b, a)
        _G['BWreagentcoloroption'].editbox:SetText(T.LABEL['reagent'])

        r, g, b, a = T.CONFIG['colorcharacter'] and unpack(T.CONFIG['colorcharacter']) or T.CHARTAB.r, T.CHARTAB.g, T.CHARTAB.b, T.CHARTAB.a
        _G['BWcharactercoloroption']:SetBackdropColor(r, g, b, a)
        _G['BWcharactercoloroption'].editbox:SetText(T.LABEL['player'])

        r, g, b, a = T.CONFIG['colorvoid'] and unpack(T.CONFIG['colorvoid']) or T.VOIDTAB.r, T.VOIDTAB.g, T.VOIDTAB.b, T.VOIDTAB.a
        _G['BWvoidcoloroption']:SetBackdropColor(r, g, b, a)
        _G['BWvoidcoloroption'].editbox:SetText(T.LABEL['void'])
        
        r, g, b, a = T.CONFIG['colorguild'] and unpack(T.CONFIG['colorguild']) or T.GUILDTAB.r, T.GUILDTAB.g, T.GUILDTAB.b, T.GUILDTAB.a
        _G['BWguildcoloroption']:SetBackdropColor(r, g, b, a)
        _G['BWguildcoloroption'].editbox:SetText(T.LABEL['guild'])

        _G['BWtooltipcoloroption']:SetBackdropColor(T.TOOLTIP.r, T.TOOLTIP.g, T.TOOLTIP.b)
        _G['BWtooltipaccentcoloroption']:SetBackdropColor(T.TOOLTIPAC.r, T.TOOLTIPAC.g, T.TOOLTIPAC.b)

        _G['BWbackgroundcoloroption']:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        _G['BWbackgroundslotcoloroption']:SetBackdropColor(T.TRANSPARENCY_SLOT.r, T.TRANSPARENCY_SLOT.g, T.TRANSPARENCY_SLOT.b, T.TRANSPARENCY_SLOT.a)
        _G['BWborderslotcoloroption']:SetBackdropColor(T.BORDER_SLOT.r, T.BORDER_SLOT.g, T.BORDER_SLOT.b, T.BORDER_SLOT.a)
        _G['BWbasecoloroption']:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
    end


    function B.C.LoadSettings()

        -- quick validation
        B.ValidateConfigs()

        T.LABEL = {
            ['bank'] = L[10001],
            ['reagent'] = L[10002],
            ['void'] = L[10003],
            ['guild'] = L[10004],
            ['player'] = L[10010],
            ['equip'] = L[10018],
            ['mail'] = L[9000],
            ['auctions'] = L[9001]
        }

        T.BANKTAB.r = 0.32; T.BANKTAB.g = 0.15; T.BANKTAB.b = 0.64; T.BANKTAB.a = 1;
        T.REAGENTTAB.r = 0.34; T.REAGENTTAB.g = 0.24; T.REAGENTTAB.b = 0.49; T.REAGENTTAB.a = 1;
        T.CHARTAB.r = 0.39; T.CHARTAB.g = 0.24; T.CHARTAB.b = 0.81; T.CHARTAB.a = 1;
        T.VOIDTAB.r = 0.24; T.VOIDTAB.g = 0.24; T.VOIDTAB.b = 0.49; T.VOIDTAB.a = 1;
        T.GUILDTAB.r = 0.20; T.GUILDTAB.g = 0.17; T.GUILDTAB.b = 0.56; T.GUILDTAB.a = 1;

        T.TOOLTIP.r = T.ITEM_QUALITY_COLORS[6].r; T.TOOLTIP.g = T.ITEM_QUALITY_COLORS[6].g; T.TOOLTIP.b = T.ITEM_QUALITY_COLORS[6].b; T.TOOLTIP.a = 1;
        T.TOOLTIPAC.r = T.ITEM_QUALITY_COLORS[5].r; T.TOOLTIPAC.g = T.ITEM_QUALITY_COLORS[5].g; T.TOOLTIPAC.b = T.ITEM_QUALITY_COLORS[5].b; T.TOOLTIPAC.a = 1;
        T.THEME.r = 0.05; T.THEME.g = 0.05; T.THEME.b = 0.05; T.THEME.a = 0.5;
        T.TRANSPARENCY.r = 0; T.TRANSPARENCY.g = 0; T.TRANSPARENCY.b = 0; T.TRANSPARENCY.a = 0.5;
        T.TRANSPARENCY_SLOT.r = 0; T.TRANSPARENCY_SLOT.g = 0; T.TRANSPARENCY_SLOT.b = 0; T.TRANSPARENCY_SLOT.a = 0.2;
        T.BORDER_SLOT.r = 0; T.BORDER_SLOT.g = 0; T.BORDER_SLOT.b = 0; T.BORDER_SLOT.a = 1;

        T.FONTS['label'] = false
        T.FONTS['quantity'] = T.FONTS[1]
        T.FONTS['quantityoutline'] = "OUTLINE"

        T.CONFIG = BWconfig['perconfig'] and BWconfig or BWsavedtable

        if T.CONFIG then
            T.BUTTON_HEIGHT = T.CONFIG['iconsize'] or 32
            T.ROW = T.CONFIG['iconrow'] or 12

            if not T.CONFIG['FontQuantity'] or T.CONFIG['FontQuantity'] == "default" then
                T.FONTS['quantity'] = T.FONTS[1]
            else
                T.FONTS['quantity'] = SML:Fetch('font', T.CONFIG['FontQuantity'])
            end

            if not T.CONFIG['FontQuantityOutline'] or T.CONFIG['FontQuantityOutline'] == "default" then
                T.FONTS['quantityoutline'] = "OUTLINE"
            else
                T.FONTS['quantityoutline'] = T.CONFIG['FontQuantityOutline']
            end

            if T.CONFIG['transparency'] then T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a = unpack(T.CONFIG['transparency']) end
            if T.CONFIG['transparencyslot'] then T.TRANSPARENCY_SLOT.r, T.TRANSPARENCY_SLOT.g, T.TRANSPARENCY_SLOT.b, T.TRANSPARENCY_SLOT.a = unpack(T.CONFIG['transparencyslot']) end
            if T.CONFIG['borderslot'] then T.BORDER_SLOT.r, T.BORDER_SLOT.g, T.BORDER_SLOT.b, T.BORDER_SLOT.a = unpack(T.CONFIG['borderslot']) end
            if T.CONFIG['basecolor'] then T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a = unpack(T.CONFIG['basecolor']) end

            if T.CONFIG['colorbank'] then T.BANKTAB.r, T.BANKTAB.g, T.BANKTAB.b, T.BANKTAB.a = unpack(T.CONFIG['colorbank']) end
            if T.CONFIG['colorbanklabel'] then T.LABEL['bank'] = T.CONFIG['colorbanklabel'] end

            if T.CONFIG['colorreagent'] then T.REAGENTTAB.r, T.REAGENTTAB.g, T.REAGENTTAB.b, T.REAGENTTAB.a = unpack(T.CONFIG['colorreagent']) end
            if T.CONFIG['colorreagentlabel'] then T.LABEL['reagent'] = T.CONFIG['colorreagentlabel'] end

            if T.CONFIG['colorcharacter'] then T.CHARTAB.r, T.CHARTAB.g, T.CHARTAB.b, T.CHARTAB.a = unpack(T.CONFIG['colorcharacter']) end
            if T.CONFIG['colorcharacterlabel'] then T.LABEL['player'] = T.CONFIG['colorcharacterlabel'] end

            if T.CONFIG['colorvoid'] then T.VOIDTAB.r, T.VOIDTAB.g, T.VOIDTAB.b, T.VOIDTAB.a = unpack(T.CONFIG['colorvoid']) end
            if T.CONFIG['colorvoidlabel'] then T.LABEL['void'] = T.CONFIG['colorvoidlabel'] end

            if T.CONFIG['colorguild'] then T.GUILDTAB.r, T.GUILDTAB.g, T.GUILDTAB.b, T.GUILDTAB.a = unpack(T.CONFIG['colorguild']) end
            if T.CONFIG['colorguildlabel'] then T.LABEL['guild'] = T.CONFIG['colorguildlabel'] end

            if T.CONFIG['colortooltip'] then T.TOOLTIP.r, T.TOOLTIP.g, T.TOOLTIP.b, T.TOOLTIP.a = unpack(T.CONFIG['colortooltip']) end
            if T.CONFIG['colortooltipaccent'] then T.TOOLTIPAC.r, T.TOOLTIPAC.g, T.TOOLTIPAC.b, T.TOOLTIPAC.a = unpack(T.CONFIG['colortooltipaccent']) end

            T.F.mainframe.bankbutton:SetText("|cffffffff"..T.LABEL['bank'].."|r")
            T.F.mainframe.reagentbutton:SetText("|cffffffff"..T.LABEL['reagent'].."|r")
            T.F.mainframe.characterbutton:SetText("|cffffffff"..T.LABEL['player'].."|r")
            T.F.mainframe.voidbutton:SetText("|cffffffff"..T.LABEL['void'].."|r")
            T.F.mainframe.guildbutton:SetText("|cffffffff"..T.LABEL['guild'].."|r")

            T.F.mainframe.bankbutton:SetBackdropColor(T.BANKTAB.r, T.BANKTAB.g, T.BANKTAB.b, T.BANKTAB.a)
            T.F.mainframe.reagentbutton:SetBackdropColor(T.REAGENTTAB.r, T.REAGENTTAB.g, T.REAGENTTAB.b, T.REAGENTTAB.a)
            T.F.mainframe.characterbutton:SetBackdropColor(T.CHARTAB.r, T.CHARTAB.g, T.CHARTAB.b, T.CHARTAB.a)
            T.F.mainframe.voidbutton:SetBackdropColor(T.VOIDTAB.r, T.VOIDTAB.g, T.VOIDTAB.b, T.VOIDTAB.a)
            T.F.mainframe.guildbutton:SetBackdropColor(T.GUILDTAB.r, T.GUILDTAB.g, T.GUILDTAB.b, T.GUILDTAB.a)
        end
    end

    function B.C.importexport(parent)

        local notes = CreateFrame("EditBox", nil, parent)
        notes:SetFontObject("ChatFontNormal")
        notes:SetWidth(150)
        notes:SetMultiLine(true)
        notes:SetAutoFocus(false)
        notes:SetScript("OnEditFocusGained", function(self) self:HighlightText(0, self:GetNumLetters()) end)
        notes:SetScript("OnEditFocusLost", function(self) self:HighlightText(0, 0) end)
        notes:SetScript("OnEscapePressed", function(self) self:ClearFocus() end)
        
        -------------- SCROLLBAR ----------------
        notes.scroll = CreateFrame('ScrollFrame', nil, parent, 'UIPanelScrollFrameTemplate')
        notes.scroll:SetPoint("TOPLEFT", 360, -70)
        notes.scroll:SetPoint("BOTTOMRIGHT", parent, "TOPLEFT", 510, -220)
        notes.scroll:SetScrollChild(notes)
        
        -------------- FAKE BACKDROP ----------------
        notes.bg = CreateFrame("EditBox", nil, parent, "InputBoxTemplate")
        notes.bg:SetAllPoints(notes.scroll)
        notes.bg:SetAutoFocus(false)
        B.I.Input(notes.bg)
        -------------- SCRIPT ----------------
        notes.bg:SetScript("OnEditFocusGained", function(self) self:GetParent().importexport:SetFocus() end)

        local button = CreateFrame("Button", nil, notes.bg, "UIPanelbuttontemplate")
        button:SetText(L[10069])
        button:SetWidth(75)
        button:SetHeight(22)
        button:SetPoint("TOPRIGHT", notes.bg, "TOPLEFT")

        button:SetScript("OnClick", B.C.Import)
        B.I.GradientButton.Create(button, "VERTICAL")
        notes.import = button

        button = CreateFrame("Button", nil, notes.bg, "UIPanelbuttontemplate")
        button:SetText(L[10070])
        button:SetWidth(75)
        button:SetHeight(22)
        button:SetPoint("TOPRIGHT", notes.import, "BOTTOMRIGHT")

        button:SetScript("OnClick", B.C.Export)
        B.I.GradientButton.Create(button, "VERTICAL")
        notes.export = button

        return notes
    end

    function B.C.Import(self)
        local k, v, x, y, setting, set_type, value
        local text = self:GetParent():GetParent().importexport:GetText()

        if not text or text == "" then return end

        local settings = {strsplit("~", text)}
        if settings[1] ~= "Bankswatch" then return end
        
        for x = 2, #settings do
            k, v = strsplit(":", settings[x])
            set_type = T.IMPORTEXPORT[k]
            if set_type then
                if v == "nil" then
                    value = nil
                elseif set_type == "table" then
                    value = {}
                    for y in string.gmatch(v, '([^--]+)') do
                        value[#value + 1] = tonumber(y)
                    end
                elseif set_type == "number" then
                    value = tonumber(v)
                elseif set_type == "boolean" then
                    value = v == "true" and true or false
                else
                    value = v
                end
                T.CONFIG[k] = value
            end
        end

        ReloadUI()
    end

    function B.C.Export(self)
        local k, x
        local str = {}
        local json = _G['json']
        str[#str + 1] = "Bankswatch"

        for k, _ in pairs(T.IMPORTEXPORT) do
            if T.CONFIG[k] and type(T.CONFIG[k]) == "table" then
                str[#str + 1] = "~"..k..":"
                for x = 1, #T.CONFIG[k] do
                    str[#str + 1] = "--"..tostring(T.CONFIG[k][x])
                end
            else
                str[#str + 1] = "~"..k..":"..tostring(T.CONFIG[k])
            end
        end

        local editbox = self:GetParent():GetParent().importexport
        editbox:SetText(table.concat(str))
        editbox:HighlightText(0, editbox:GetNumLetters())
    end

    function B.C.perconfigcheckbox(parent)

        local checkbox = CreateFrame("CheckButton", "BWperconfigcheckbox", parent, "InterfaceOptionsCheckButtonTemplate")
        checkbox:SetPoint("TOPLEFT", 23, -70)
        checkbox:SetHitRectInsets(0, -300, 0, 0)
        _G['BWperconfigcheckboxText']:SetText(L[10061])
        checkbox:SetChecked(BWconfig['perconfig'] or false)
        checkbox:SetScript("OnClick", function(self) 
            if self:GetChecked() then
                BWconfig['perconfig'] = true
            else
                BWconfig['perconfig'] = false
            end
            B.C.LoadSettings()
            B.AdjustTransparency()
            B.ResetFontButtons()
            self:GetParent().refresh()
        end)
        return checkbox
    end

    function B.C.iconsizeslider(parent)

        local slider = CreateFrame("Slider", "BWiconsizeslider", parent, "OptionsSliderTemplate")
        slider:SetWidth(200)
        slider:SetHeight(16)
        slider:SetPoint("TOPLEFT", 25, -120)
        _G['BWiconsizesliderLow']:SetText("27")
        _G['BWiconsizesliderHigh']:SetText("40")
        slider:SetMinMaxValues(27, 40)
        slider:SetValueStep(1)
        slider:SetValue(T.CONFIG['iconsizeslider'] and tonumber(T.CONFIG['iconsizeslider']) or 32)
        _G['BWiconsizesliderText']:SetFormattedText(L[10030]..": %d ("..L[9015]..": 32)", slider:GetValue())
        slider:SetScript("OnValueChanged", function(self, value)
            value = math.floor(value)
            _G['BWiconsizesliderText']:SetFormattedText(L[10030]..": %d ("..L[9015]..": 32)", value)
            T.CONFIG['iconsize'] = value
            T.CONFIG['iconsizeslider'] = value
            T.BUTTON_HEIGHT = value
        end)
        return slider
    end

    function B.C.iconrowslider(parent)

        local slider = CreateFrame("Slider", "BWiconrowslider", parent, "OptionsSliderTemplate")
        slider:SetWidth(200)
        slider:SetHeight(16)
        slider:SetPoint("TOPLEFT", 25, -170)
        _G['BWiconrowsliderLow']:SetText("10")
        _G['BWiconrowsliderHigh']:SetText("20")
        slider:SetMinMaxValues(10, 20)
        slider:SetValueStep(1)
        slider:SetValue(T.CONFIG['iconrowslider'] and tonumber(T.CONFIG['iconrowslider']) or 12)
        _G['BWiconrowsliderText']:SetFormattedText(L[10031]..": %d ("..L[9015]..": 12)", slider:GetValue())
        slider:SetScript("OnValueChanged", function(self, value)
            _G['BWiconrowsliderText']:SetFormattedText(L[10031]..": %d ("..L[9015]..": 12)", value)
            T.CONFIG['iconrow'] = math.floor(value + 0.5)
            T.CONFIG['iconrowslider'] = value
        end)
        return slider
    end

    function B.C.reloaduibutton(parent, text)

        local label = parent:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
        label:SetPoint("TOPLEFT", parent, 25, -210)
        label:SetText("*"..L[10032])
        label:SetTextColor(1, 0, 0)

        local button = CreateFrame("Button", nil, parent, "UIPanelbuttontemplate")
        button:SetText(text)
        button:SetWidth(100)
        button:SetHeight(32)
        button:SetPoint("TOPLEFT", 25, -225)
        button.label = label

        button:SetScript("OnClick", function(self) ReloadUI() end)
        B.I.GradientButton.Create(button, "VERTICAL")
        return button
    end

    function B.C.minimizecheckbox(parent)

        local checkbox = CreateFrame("CheckButton", "BWminimizecheckbox", parent, "InterfaceOptionsCheckButtonTemplate")
        checkbox:SetPoint("TOPLEFT", 23, -270)
        checkbox:SetHitRectInsets(0, -300, 0, 0)
        _G['BWminimizecheckboxText']:SetText(L[10033])
        checkbox:SetChecked(T.CONFIG['minimize'] or false)
        checkbox:SetScript("OnClick", function(self) 
            if self:GetChecked() then
                T.CONFIG['minimize'] = true
            else
                T.CONFIG['minimize'] = false
            end
        end)
        return checkbox
    end

    function B.C.openbankcheckbox(parent)

        local checkbox = CreateFrame("CheckButton", "BWopenbankcheckbox", parent, "InterfaceOptionsCheckButtonTemplate")
        checkbox:SetPoint("TOPLEFT", 23, -295)
        checkbox:SetHitRectInsets(0, -300, 0, 0)
        _G['BWopenbankcheckboxText']:SetText(L[10036])
        checkbox:SetChecked(T.CONFIG['openbank'] or false)
        checkbox:SetScript("OnClick", function(self) 
            if self:GetChecked() then
                T.CONFIG['openbank'] = true
            else
                T.CONFIG['openbank'] = false
            end
        end)
        return checkbox
    end

    function B.C.openplayercheckbox(parent)

        local checkbox = CreateFrame("CheckButton", "BWopenplayercheckbox", parent, "InterfaceOptionsCheckButtonTemplate")
        checkbox:SetPoint("TOPLEFT", 23, -320)
        checkbox:SetHitRectInsets(0, -300, 0, 0)
        _G['BWopenplayercheckboxText']:SetText(L[10037])
        checkbox:SetChecked(T.CONFIG['openplayer'] or false)
        checkbox:SetScript("OnClick", function(self) 
            if self:GetChecked() then
                T.CONFIG['openplayer'] = true
            else
                T.CONFIG['openplayer'] = false
            end
        end)
        return checkbox
    end


    B.C.FontQuantityLabel = {}
    function B.C.FontQuantityLabel.Create(parent)

        local menu = CreateFrame("Frame", "BWquantityfontdropDownMenu", parent, "L_UIDropDownMenuTemplate")
        L_UIDropDownMenu_SetWidth(menu, 150, 4)
        menu:SetPoint("TOPLEFT", 10, -370)
        menu.Text:SetWidth(menu.Text:GetWidth() - 15)

        local font = T.CONFIG['FontQuantity'] or "default"
        originalfont, originalsize, originaloutline = _G['BWquantityfontdropDownMenuText']:GetFont()

        L_UIDropDownMenu_Initialize(_G['BWquantityfontdropDownMenu'], B.C.Font.Initialize) --The virtual
        L_UIDropDownMenu_SetSelectedValue(_G['BWquantityfontdropDownMenu'], font)
        B.C.Font_OnChanged(_G['BWquantityfontdropDownMenu'], font, font)

        local text = menu:CreateFontString("BWquantityfontdropLabel", "OVERLAY", "GameFontNormalSmall")
        text:SetPoint("BOTTOMLEFT", menu, "TOPLEFT", 16, 3)
        text:SetText(L[10063])
        menu.text = text

        return menu
    end

    local fontlist
    B.C.Font = {}
    function B.C.Font:Initialize(level)
        --drop down menues can have sub menues. The value of level determines the drop down sub menu tier
        local level = level or 1 
        local k, v, button, normalText, size, outline, font

        local listFrame = _G['L_DropDownList1']
        local listFrameName = listFrame:GetName();
        local config = T.CONFIG['FontQuantity']

        local info = L_UIDropDownMenu_CreateInfo()

        B.C.Font.Button(info, L[9015], "default", config == "default" and true or config == nil and true or false, B.C.Font_OnClick)

        fontlist = fontlist or SML:HashTable("font")
        for k, v in pairs(fontlist) do
            B.C.Font.Button(info, k, k, config == k, B.C.Font_OnClick)

            button = _G[listFrameName.."Button"..listFrame.numButtons]
            normalText = _G[button:GetName().."NormalText"]

            font = fontlist[k] ~= k and fontlist[k] or SML:Fetch('font', k)
            normalText:SetFont(font, originalsize, originaloutline)
            button:SetScript("OnHide", B.C.ResetButton)
        end
    end

    function B.C.Font.Button(info, label, value, checked, func)

        info.text = label
        info.hasArrow = false
        info.value = value
        info.func = func
        info.owner = _G['BWquantityfontdropDownMenu']
        info.checked = checked

        L_UIDropDownMenu_AddButton(info, 1)
    end

    function B.C.Font_OnClick(self)

        local parent = self:GetParent()
        local value = self.value

        T.CONFIG["FontQuantity"] = value
        B.C.Font_OnChanged(parent.dropdown, value)
    end

    function B.C.Font_OnChanged(dropdown, value)
        local font

        if value == "default" then
            font = originalfont
        else
            font = SML:Fetch('font', value)
        end

        T.FONTS['quantity'] = value == "default" and T.FONTS[1] or font
        dropdown.Text:SetFont(font, originalsize, originaloutline)
        L_UIDropDownMenu_SetSelectedValue(dropdown, value)
        B.ResetFontButtons()
    end

    B.C.FontOutlineLabel = {}
    function B.C.FontOutlineLabel.Create(parent)

        local menu = CreateFrame("Frame", "BWquantityoutlinedropDownMenu", parent, "UIDropDownMenuTemplate")
        UIDropDownMenu_SetWidth(menu, 150, 4)
        menu:SetPoint("TOPLEFT", 180, -370)
        menu.Text:SetWidth(menu.Text:GetWidth() - 15)

        UIDropDownMenu_Initialize(_G['BWquantityoutlinedropDownMenu'], B.C.FontOutlineLabel.Initialize) --The virtual
        UIDropDownMenu_SetSelectedValue(_G['BWquantityoutlinedropDownMenu'], T.CONFIG['FontQuantityOutline'] or "OUTLINE")

        local text = menu:CreateFontString("BWquantityoutlinedropLabel", "OVERLAY", "GameFontNormalSmall")
        text:SetPoint("BOTTOMLEFT", menu, "TOPLEFT", 16, 3)
        text:SetText(L[10064])
        menu.text = text

        return menu
    end

    function B.C.FontOutlineLabel:Initialize(level)
        --drop down menues can have sub menues. The value of level determines the drop down sub menu tier
        local level = level or 1 
        local k, v, button, normalText, size, outline, font

        local config = T.CONFIG['FontQuantityOutline']

        local info = UIDropDownMenu_CreateInfo()

        for k, _ in pairs(T.FLAGS) do
            B.C.FontOutlineLabel.Button(info, k, k, config == k, B.C.FontOutline_OnClick)
        end
    end

    function B.C.FontOutline_OnClick(self)

        local parent = self:GetParent()
        local value = self.value

        T.CONFIG['FontQuantityOutline'] = value
        T.FONTS['quantityoutline'] = value
        UIDropDownMenu_SetSelectedValue(parent.dropdown, T.CONFIG['FontQuantityOutline'])
        B.ResetFontButtons()
    end

    function B.C.FontOutlineLabel.Button(info, label, value, checked, func)

        info.text = label
        info.hasArrow = false
        info.value = value
        info.func = func
        info.owner = _G['BWquantityoutlinedropDownMenu']
        info.checked = checked

        UIDropDownMenu_AddButton(info, 1)
    end

    function B.C.colorselect(parent)

        local colorbox = CreateFrame("Frame", nil, parent)
        colorbox:SetAllPoints()
        colorbox:SetPoint("TOPLEFT", 30, -400)

        B.C.ColorBank.Create(colorbox, 1)
        B.C.ColorReagent.Create(colorbox, 2)
        B.C.ColorCharacter.Create(colorbox, 3)
        B.C.ColorVoid.Create(colorbox, 4)
        B.C.ColorGuild.Create(colorbox, 5)

        B.C.ColorBackground.Create(colorbox, 2)
        B.C.ColorBase.Create(colorbox, 1)
        B.C.ColorBackgroundSlot.Create(colorbox, 3)
        B.C.ColorBorderSlot.Create(colorbox, 4)

        B.C.ColorTooltip.Create(colorbox, 1)
        B.C.ColorTooltipAccent.Create(colorbox, 2)

        return colorbox
    end

    B.C.ColorButton = {}
    function B.C.ColorButton.Create(parent, name)

        local button = CreateFrame("Button", name, parent)
        button:SetWidth(21)
        button:SetHeight(21)

        B.I.Backdrop(button)
        button:SetBackdropBorderColor(1, 1, 1, 0.3)

        return button
    end

    B.C.ColorLabel = {}
    function B.C.ColorLabel.Create(parent)

        local editbox = CreateFrame("EditBox", nil, parent, "InputBoxTemplate")

        editbox:SetPoint("TOPLEFT", parent, "TOPRIGHT", 15, 0)
        editbox:SetAutoFocus(false)
        editbox:SetMultiLine(true) -- make text show up
        editbox:SetTextColor(1, 1, 1, 1)
        editbox:SetTextInsets(5, 5, 4, 4)
        editbox:SetWidth(30)

        B.I.RemoveEdge(editbox)
        B.I.Backdrop(editbox)
        editbox:SetBackdropBorderColor(1, 1, 1, 0.3)

        parent.editbox = editbox
        return editbox
    end

    function B.C.GetColor(restore)
        local r, g, b, a

        if restore then
            r, g, b, a = unpack(restore)
        else
            a, r, g, b = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB()
        end

        return r, g, b, a
    end

    B.C.ColorBank = {}
    function B.C.ColorBank.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWbankcoloroption")
        button:SetPoint("TOPLEFT", 0, -button:GetHeight() * index)
        button:SetBackdropColor(T.BANKTAB.r, T.BANKTAB.g, T.BANKTAB.b, T.BANKTAB.a)
        button:SetScript("OnClick", B.C.ColorBank.Click)
        parent.colorbank = button

        local label = B.C.ColorLabel.Create(button)
        label:SetHeight(button:GetHeight())
        label:SetMaxLetters(1)
        label:SetText(T.LABEL['bank'])
        label:SetScript("OnTextChanged", B.C.ColorBank.TextChanged)
    end

    function B.C.ColorBank:TextChanged(userInput)
        local text = self:GetText()
        text = text ~= "" and false or self:GetText()

        if userInput and text then
            T.LABEL['bank'] = text
            T.F.mainframe.bankbutton:SetText("|cffffffff"..text.."|r")
            T.CONFIG['colorbanklabel'] = text
        end
    end

    function B.C.ColorBank:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorBank.ColorChanged)
    end

    function B.C.ColorBank.ColorChanged(restore)
        local r, g, b, a = B.C.GetColor(restore)
        -- Update
        T.F.mainframe.bankbutton:SetBackdropColor(r, g, b, a)
        _G['BWbankcoloroption']:SetBackdropColor(r, g, b, a)
        T.CONFIG['colorbank'] = {r, g, b, a}
        T.BANKTAB.r = r; T.BANKTAB.g = g; T.BANKTAB.b = b; T.BANKTAB.a = a
    end

    B.C.ColorReagent = {}
    function B.C.ColorReagent.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWreagentcoloroption")
        button:SetPoint("TOPLEFT", 0, -button:GetHeight() * index)
        button:SetBackdropColor(T.REAGENTTAB.r, T.REAGENTTAB.g, T.REAGENTTAB.b, T.REAGENTTAB.a)
        button:SetScript("OnClick", B.C.ColorReagent.Click)
        parent.colorreagent = button

        local label = B.C.ColorLabel.Create(button)
        label:SetHeight(button:GetHeight())
        label:SetMaxLetters(1)
        label:SetText(T.LABEL['reagent'])
        label:SetScript("OnTextChanged", B.C.ColorReagent.TextChanged)
    end

    function B.C.ColorReagent:TextChanged(userInput)
        local text = self:GetText()
        text = text ~= "" and false or self:GetText()

        if userInput and text then
            T.LABEL['reagent'] = text
            T.F.mainframe.reagentbutton:SetText("|cffffffff"..text.."|r")
            T.CONFIG['colorreagentlabel'] = text
        end
    end

    function B.C.ColorReagent:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorReagent.ColorChanged)
    end

    function B.C.ColorReagent.ColorChanged(restore)
        local r, g, b, a = B.C.GetColor(restore)
        -- Update
        T.F.mainframe.reagentbutton:SetBackdropColor(r, g, b, a)
        _G['BWreagentcoloroption']:SetBackdropColor(r, g, b, a)
        T.CONFIG['colorreagent'] = {r, g, b, a}
        T.REAGENTTAB.r = r; T.REAGENTTAB.g = g; T.REAGENTTAB.b = b; T.REAGENTTAB.a = a
    end

    B.C.ColorCharacter = {}
    function B.C.ColorCharacter.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWcharactercoloroption")
        button:SetPoint("TOPLEFT", 0, -button:GetHeight() * index)
        button:SetBackdropColor(T.CHARTAB.r, T.CHARTAB.g, T.CHARTAB.b, T.CHARTAB.a)
        button:SetScript("OnClick", B.C.ColorCharacter.Click)
        parent.colorcharacter = button

        local label = B.C.ColorLabel.Create(button)
        label:SetHeight(button:GetHeight())
        label:SetMaxLetters(1)
        label:SetText(T.LABEL['player'])
        label:SetScript("OnTextChanged", B.C.ColorCharacter.TextChanged)
    end

    function B.C.ColorCharacter:TextChanged(userInput)
        local text = self:GetText()
        text = text ~= "" and false or self:GetText()

        if userInput and text then
            T.LABEL['player'] = text
            T.F.mainframe.characterbutton:SetText("|cffffffff"..text.."|r")
            T.CONFIG['colorcharacterlabel'] = text
        end
    end

    function B.C.ColorCharacter:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorCharacter.ColorChanged)
    end

    function B.C.ColorCharacter.ColorChanged(restore)
        local r, g, b, a = B.C.GetColor(restore)
        -- Update
        T.F.mainframe.characterbutton:SetBackdropColor(r, g, b, a)
        _G['BWcharactercoloroption']:SetBackdropColor(r, g, b, a)
        T.CONFIG['colorcharacter'] = {r, g, b, a}
        T.CHARTAB.r = r; T.CHARTAB.g = g; T.CHARTAB.b = b; T.CHARTAB.a = a
    end

    B.C.ColorVoid = {}
    function B.C.ColorVoid.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWvoidcoloroption")
        button:SetPoint("TOPLEFT", 0, -button:GetHeight() * index)
        button:SetBackdropColor(T.VOIDTAB.r, T.VOIDTAB.g, T.VOIDTAB.b, T.VOIDTAB.a)
        button:SetScript("OnClick", B.C.ColorVoid.Click)
        parent.colorvoid = button

        local label = B.C.ColorLabel.Create(button)
        label:SetHeight(button:GetHeight())
        label:SetMaxLetters(1)
        label:SetText(T.LABEL['void'])
        label:SetScript("OnTextChanged", B.C.ColorVoid.TextChanged)
    end

    function B.C.ColorVoid:TextChanged(userInput)
        local text = self:GetText()
        text = text ~= "" and false or self:GetText()

        if userInput and text then
            T.LABEL['void'] = text
            T.F.mainframe.voidbutton:SetText("|cffffffff"..text.."|r")
            T.CONFIG['colorvoidlabel'] = text
        end
    end

    function B.C.ColorVoid:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorVoid.ColorChanged)
    end

    function B.C.ColorVoid.ColorChanged(restore)
        local r, g, b, a = B.C.GetColor(restore)
        -- Update
        T.F.mainframe.voidbutton:SetBackdropColor(r, g, b, a)
        _G['BWvoidcoloroption']:SetBackdropColor(r, g, b, a)
        T.CONFIG['colorvoid'] = {r, g, b, a}
        T.VOIDTAB.r = r; T.VOIDTAB.g = g; T.VOIDTAB.b = b; T.VOIDTAB.a = a
    end

    B.C.ColorGuild = {}
    function B.C.ColorGuild.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWguildcoloroption")
        button:SetPoint("TOPLEFT", 0, -button:GetHeight() * index)
        button:SetBackdropColor(T.GUILDTAB.r, T.GUILDTAB.g, T.GUILDTAB.b, T.GUILDTAB.a)
        button:SetScript("OnClick", B.C.ColorGuild.Click)
        parent.colorguild = button

        local label = B.C.ColorLabel.Create(button)
        label:SetHeight(button:GetHeight())
        label:SetMaxLetters(1)
        label:SetText(T.LABEL['guild'])
        label:SetScript("OnTextChanged", B.C.ColorGuild.TextChanged)
    end

    function B.C.ColorGuild:TextChanged(userInput)
        local text = self:GetText()
        text = text ~= "" and false or self:GetText()

        if userInput and text then
            T.LABEL['guild'] = text
            T.F.mainframe.guildbutton:SetText("|cffffffff"..text.."|r")
            T.CONFIG['colorguildlabel'] = text
        end
    end

    function B.C.ColorGuild:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorGuild.ColorChanged)
    end

    function B.C.ColorGuild.ColorChanged(restore)
        local r, g, b, a = B.C.GetColor(restore)
        -- Update
        T.F.mainframe.guildbutton:SetBackdropColor(r, g, b, a)
        _G['BWguildcoloroption']:SetBackdropColor(r, g, b, a)
        T.CONFIG['colorguild'] = {r, g, b, a}
        T.GUILDTAB.r = r; T.GUILDTAB.g = g; T.GUILDTAB.b = b; T.GUILDTAB.a = a
    end

    B.C.ColorTooltip = {}
    function B.C.ColorTooltip.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWtooltipcoloroption")
        button:SetPoint("TOPLEFT", 340, -button:GetHeight() * index)
        button:SetBackdropColor(T.TOOLTIP.r, T.TOOLTIP.g, T.TOOLTIP.b, 1)
        button:SetScript("OnClick", B.C.ColorTooltip.Click)
        parent.colortooltip = button

        local label = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        label:SetText("|cffffffff"..L[10034].."|r")
        label:SetPoint("LEFT", 30, 0)
        button.label = label
    end

    function B.C.ColorTooltip:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorTooltip.ColorChanged)
    end

    function B.C.ColorTooltip.ColorChanged(restore)
        local r, g, b = B.C.GetColor(restore)
        -- Update
        _G['BWtooltipcoloroption']:SetBackdropColor(r, g, b, 1)
        T.CONFIG['colortooltip'] = {r, g, b, 1}
        T.TOOLTIP.r = r; T.TOOLTIP.g = g; T.TOOLTIP.b = b;
    end

    B.C.ColorTooltipAccent = {}
    function B.C.ColorTooltipAccent.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWtooltipaccentcoloroption")
        button:SetPoint("TOPLEFT", 340, -button:GetHeight() * index)
        button:SetBackdropColor(T.TOOLTIPAC.r, T.TOOLTIPAC.g, T.TOOLTIPAC.b, 1)
        button:SetScript("OnClick", B.C.ColorTooltipAccent.Click)
        parent.colortooltipaccent = button

        local label = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        label:SetText("|cffffffff"..L[10035].."|r")
        label:SetPoint("LEFT", 30, 0)
        button.label = label
    end

    function B.C.ColorTooltipAccent:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorTooltipAccent.ColorChanged)
    end

    function B.C.ColorTooltipAccent.ColorChanged(restore)
        local r, g, b = B.C.GetColor(restore)
        -- Update
        _G['BWtooltipaccentcoloroption']:SetBackdropColor(r, g, b, 1)
        T.CONFIG['colortooltipaccent'] = {r, g, b, 1}
        T.TOOLTIPAC.r = r; T.TOOLTIPAC.g = g; T.TOOLTIPAC.b = b;
    end

    B.C.ColorBackground = {}
    function B.C.ColorBackground.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWbackgroundcoloroption")
        button:SetPoint("TOPLEFT", 120, -button:GetHeight() * index)
        button:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        button:SetScript("OnClick", B.C.ColorBackground.Click)
        parent.transparency = button

        local label = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        label:SetText("|cffffffff"..L[9016].."|r")
        label:SetPoint("LEFT", 30, 0)
        button.label = label
    end

    function B.C.ColorBackground:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorBackground.ColorChanged)
    end

    function B.C.ColorBackground.ColorChanged(restore)
        local r, g, b, a = B.C.GetColor(restore)
        -- Update
        _G['BWbackgroundcoloroption']:SetBackdropColor(r, g, b, a)
        T.CONFIG['transparency'] = {r, g, b, a}
        T.TRANSPARENCY.r = r; T.TRANSPARENCY.g = g; T.TRANSPARENCY.b = b; T.TRANSPARENCY.a = a;
        B.AdjustTransparency()
    end

    B.C.ColorBackgroundSlot = {}
    function B.C.ColorBackgroundSlot.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWbackgroundslotcoloroption")
        button:SetPoint("TOPLEFT", 120, -button:GetHeight() * index)
        button:SetBackdropColor(T.TRANSPARENCY_SLOT.r, T.TRANSPARENCY_SLOT.g, T.TRANSPARENCY_SLOT.b, T.TRANSPARENCY_SLOT.a)
        button:SetScript("OnClick", B.C.ColorBackgroundSlot.Click)
        parent.transparency = button

        local label = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        label:SetText("|cffffffff"..L[10065].."|r")
        label:SetPoint("LEFT", 30, 0)
        button.label = label
    end

    function B.C.ColorBackgroundSlot:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorBackgroundSlot.ColorChanged)
    end

    function B.C.ColorBackgroundSlot.ColorChanged(restore)
        local r, g, b, a = B.C.GetColor(restore)
        -- Update
        _G['BWbackgroundslotcoloroption']:SetBackdropColor(r, g, b, a)
        T.CONFIG['transparencyslot'] = {r, g, b, a}
        T.TRANSPARENCY_SLOT.r = r; T.TRANSPARENCY_SLOT.g = g; T.TRANSPARENCY_SLOT.b = b; T.TRANSPARENCY_SLOT.a = a;
        B.AdjustTransparency()
    end

    B.C.ColorBorderSlot = {}
    function B.C.ColorBorderSlot.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWborderslotcoloroption")
        button:SetPoint("TOPLEFT", 120, -button:GetHeight() * index)
        button:SetBackdropColor(T.BORDER_SLOT.r, T.BORDER_SLOT.g, T.BORDER_SLOT.b, T.BORDER_SLOT.a)
        button:SetScript("OnClick", B.C.ColorBorderSlot.Click)
        parent.transparency = button

        local label = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        label:SetText("|cffffffff"..L[10068].."|r")
        label:SetPoint("LEFT", 30, 0)
        button.label = label
    end

    function B.C.ColorBorderSlot:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorBorderSlot.ColorChanged)
    end

    function B.C.ColorBorderSlot.ColorChanged(restore)
        local r, g, b, a = B.C.GetColor(restore)
        -- Update
        _G['BWborderslotcoloroption']:SetBackdropColor(r, g, b, a)
        T.CONFIG['borderslot'] = {r, g, b, a}
        T.BORDER_SLOT.r = r; T.BORDER_SLOT.g = g; T.BORDER_SLOT.b = b; T.BORDER_SLOT.a = a;
        B.AdjustTransparency()
    end

    B.C.ColorBase = {}
    function B.C.ColorBase.Create(parent, index)

        local button = B.C.ColorButton.Create(parent, "BWbasecoloroption")
        button:SetPoint("TOPLEFT", 120, -button:GetHeight() * index)
        button:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        button:SetScript("OnClick", B.C.ColorBase.Click)
        parent.transparency = button

        local label = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        label:SetText("|cffffffff".."Base color".."|r")
        label:SetPoint("LEFT", 30, 0)
        button.label = label
    end

    function B.C.ColorBase:Click(...)
        local r, g, b, a = self:GetBackdropColor()
        B.I.ColorPicker.Create(r, g, b, a, B.C.ColorBase.ColorChanged)
    end

    function B.C.ColorBase.ColorChanged(restore)
        local r, g, b, a = B.C.GetColor(restore)
        -- Update
        _G['BWbasecoloroption']:SetBackdropColor(r, g, b, a)
        T.CONFIG['basecolor'] = {r, g, b, a}
        T.THEME.r = r; T.THEME.g = g; T.THEME.b = b; T.THEME.a = a;
        B.AdjustTransparency()
    end