-- GLOBALS: unpack, select, LibStub, ITEM_SEARCHBAR_LIST
-- GLOBALS: VoidStorageFrame, ReagentBankFrame, MerchantFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _G = _G
    local math = math
    local tostring = tostring
    local tonumber = tonumber
    local string = string
    local pairs = pairs
    local format = format
    local tinsert = tinsert
    local strmatch = strmatch
    local strsplit = strsplit

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local ResetCursor = ResetCursor
    local SetCursor = SetCursor
    local ShowInspectCursor = ShowInspectCursor
    local GameTooltip = GameTooltip
    local GameTooltip_Hide = GameTooltip_Hide
    local GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem
    local BattlePetToolTip_Show = BattlePetToolTip_Show
    local BattlePetTooltip = BattlePetTooltip
    local CloseDropDownMenus = CloseDropDownMenus
    local ToggleDropDownMenu = ToggleDropDownMenu
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UseContainerItem = UseContainerItem
    local PickupContainerItem = PickupContainerItem
    local PickupGuildBankItem = PickupGuildBankItem
    local GetItemQualityColor = GetItemQualityColor
    local SplitContainerItem = SplitContainerItem
    local SplitGuildBankItem = SplitGuildBankItem
    local OpenStackSplitFrame = OpenStackSplitFrame
    local ClickVoidStorageSlot = ClickVoidStorageSlot
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local AutoStoreGuildBankItem = AutoStoreGuildBankItem
    local GetCVarBool = GetCVarBool
    local IsShiftKeyDown = IsShiftKeyDown
    local IsModifiedClick = IsModifiedClick
    local IsControlKeyDown = IsControlKeyDown
    local HandleModifiedItemClick = HandleModifiedItemClick
    local SocketContainerItem = SocketContainerItem
    local GetMoneyString = GetMoneyString
    local ColorPickerFrame = ColorPickerFrame
    local FauxScrollFrame_OnVerticalScroll = FauxScrollFrame_OnVerticalScroll
    local IsBagOpen, OpenBag = IsBagOpen, OpenBag
    local IsArtifactPowerItem = IsArtifactPowerItem
    local SocketInventoryItem = SocketInventoryItem
    local SpellCanTargetItem = SpellCanTargetItem
    local SpellCanTargetItemID = SpellCanTargetItemID
    local DressUpItemLink = DressUpItemLink

    local B, L, T = unpack(select(2, ...))

    -- forward declaration
    local create_bank, create_reagent, create_character, create_void, create_guild, create_search

    B.I =  {['bank'] = {['num'] = 0, ['rows'] = 0},
            ['character'] = {['num'] = 0, ['rows'] = 0},
            ['OptionsDropDown'] = {},
            ['Bank'] = {['Bags'] = {}, ['Buttons'] = {}, ['Tooltip'] = {}, ['BagToggle'] = {}},
            ['Character'] = {['Bags'] = {}, ['Buttons'] = {}, ['Tab'] = {}, ['Tooltip'] = {}, ['BagToggle'] = {},
                             ['Artifact'] = {}, ['Equipment'] = {}, ['Auctions'] = {['Button'] = {}},
                             ['Mail'] = {['Button'] = {}}},
            ['Reagent'] = {},
            ['Void'] = {['Tabs'] = {}},
            ['Guild'] = {['Tabs'] = {}},
            ['Search'] = {['Count'] = {}, ['Button'] = {}},
            ['Toggle'] = {},
            ['Options'] = {},
            ['Main'] = {['Frame'] = {}}
            }


--[[//////////////////////////////////////////////////

    MAIN INTERFACE FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    local function create_status() -- just texture that tracks the status of the bar

        local status = T.F.toggleframe:CreateTexture(nil, "OVERLAY")
        status:SetHeight(18)
        status:SetWidth(18)
        status:SetTexture(T.icon)
        status:SetVertexColor(1, 1, 1, 1)
        status:SetBlendMode("ADD")
        status:SetPoint("CENTER", T.F.mainframe.optionbutton, -0.5, 0.5)
        status:SetParent(T.F.mainframe.optionbutton)

        T.F.status = status
    end

    local function create_togglebutton()

        local button = B.I.Main.Frame.Button(nil, T.F.toggleframe)
        button:SetHighlightTexture("")
        button:SetAllPoints(T.F.mainframe.optionbutton)
        button:SetClampedToScreen(true)
        button:SetBackdropColor(0.07, 0.07, 0.07, 0.8)
        button:SetBackdropBorderColor(0.15, 0.15, 0.15, 1)
        button:Hide()

        button:SetScript("OnEnter", B.I.Toggle.Enter)
        button:SetScript("OnLeave", B.I.Toggle.Leave)
        button:SetScript("OnShow", B.I.Toggle.Show)
        button:SetScript("OnHide", B.I.Toggle.Hide)
        button:SetScript("OnClick", B.I.Toggle.Click)

        T.F.mainframe.togglebutton = button
    end

    local function create_optionbutton()

        local button = B.I.Main.Frame.Button(nil, nil, "BWoptionbutton")
        button:SetHighlightTexture("")
        button:SetPoint("TOPLEFT", T.F.mainframe, "TOPLEFT")
        button:SetPoint("BOTTOM", T.F.mainframe, "BOTTOM")
        button:SetBackdropColor(0.07, 0.07, 0.07, 1)
        button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        button:RegisterForDrag("LeftButton")

        button:SetScript("OnShow", B.I.Options.Show)
        button:SetScript("OnEnter", B.I.Options.Enter)
        button:SetScript("OnLeave", B.I.Options.Leave)
        button:SetScript("OnDragStart", B.I.Options.DragStart)
        button:SetScript("OnDragStop", B.I.Options.DragStop)
        button:SetScript("OnClick", B.I.Options.Click)

        B.I.OptionsDropDown.Create()
        T.F.mainframe.optionbutton = button

        create_togglebutton()
        create_status()
    end

    local function create_bankbutton()

        local button = B.I.Main.Frame.Button("|cffffffff"..L[10001].."|r")
        button:SetPoint("LEFT", T.F.mainframe.optionbutton, "RIGHT")
        button:SetBackdropColor(T.BANKTAB.r, T.BANKTAB.g, T.BANKTAB.b, T.BANKTAB.a)

        button:SetScript("OnClick", B.I.Bank.Click)

        T.F.mainframe.bankbutton = button
    end

    local function create_reagentbutton()

        local button = B.I.Main.Frame.Button("|cffffffff"..L[10002].."|r")
        button:SetPoint("LEFT", T.F.mainframe.bankbutton, "RIGHT")
        button:SetBackdropColor(T.REAGENTTAB.r, T.REAGENTTAB.g, T.REAGENTTAB.b, T.REAGENTTAB.a)

        button:SetScript("OnClick", B.I.Reagent.Click)

        T.F.mainframe.reagentbutton = button
    end

    local function create_characterbutton()

        local button = B.I.Main.Frame.Button("|cffffffff"..L[10010].."|r")
        button:SetPoint("LEFT", T.F.mainframe.reagentbutton, "RIGHT")
        button:SetBackdropColor(T.CHARTAB.r, T.CHARTAB.g, T.CHARTAB.b, T.CHARTAB.a)

        button:SetScript("OnClick", B.I.Character.Click)

        T.F.mainframe.characterbutton = button
    end

    local function create_voidbutton()

        local button = B.I.Main.Frame.Button("|cffffffff"..L[10003].."|r")
        button:SetPoint("LEFT", T.F.mainframe.characterbutton, "RIGHT")
        button:SetBackdropColor(T.VOIDTAB.r, T.VOIDTAB.g, T.VOIDTAB.b, T.VOIDTAB.a)

        button:SetScript("OnClick", B.I.Void.Click)

        T.F.mainframe.voidbutton = button
    end

    local function create_guildbutton()

        local button = B.I.Main.Frame.Button("|cffffffff"..L[10004].."|r")
        button:SetPoint("LEFT", T.F.mainframe.voidbutton, "RIGHT")
        button:SetBackdropColor(T.GUILDTAB.r, T.GUILDTAB.g, T.GUILDTAB.b, T.GUILDTAB.a)

        button:SetScript("OnClick", B.I.Guild.Click)
        button:HookScript("OnEnter", B.I.Guild.Enter)
        button:HookScript("OnLeave", B.I.Guild.Leave)

        T.F.mainframe.guildbutton = button
    end


    local function create_toggleframe()

        local frame = CreateFrame("Frame", "BWtoggleframe", UIParent)
        frame:SetAllPoints()

        frame:SetScript("OnShow", function(self) BWconfig['state'] = 1 end)
        frame:SetScript("OnHide", function(self) BWconfig['state'] = 0 end)

        T.F.toggleframe = frame
    end

    local function create_mainframe()

        create_toggleframe()

        local frame = CreateFrame("Frame", "BWmainframe", T.F.toggleframe)
        frame:SetPoint("CENTER")
        frame:SetHeight(25)
        frame:SetWidth(15 + (32 * 6) + 192) -- options + searchbar
        frame:SetFrameStrata("DIALOG")
        frame:SetMovable(true)
        frame:SetUserPlaced(true)
        frame:SetClampedToScreen(true)
        frame.layout = 0

        frame:SetScript("OnMouseDown", function(self) self:StartMoving() end)
        frame:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() end)
        frame:SetScript("OnHide", function(self) CloseDropDownMenus() end)

        B.I.Backdrop(frame)
        T.F.mainframe = frame

        -- Create option button which is the anchor
        create_optionbutton()

        create_bankbutton()
        create_reagentbutton()
        create_characterbutton()
        create_voidbutton()
        create_guildbutton()
    end

    local function create_bankframe()

        local frame = CreateFrame("Frame", "BWbank", T.F.mainframe)
        frame:SetPoint("TOPLEFT", T.F.mainframe, "BOTTOMLEFT", 0, 1)
        frame:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        frame:SetFrameStrata("HIGH")

        frame:SetScript("OnShow", function(self) T.F.mainframe.bankbutton:LockHighlight() end)
        frame:SetScript("OnHide", function(self) T.F.mainframe.bankbutton:UnlockHighlight() end)

        B.I.Backdrop(frame)
        T.F.mainframe.bank = frame
    end

    local function create_reagentframe()

        local frame = CreateFrame("Frame", "BWreagent", T.F.mainframe)
        frame:SetPoint("TOPLEFT", T.F.mainframe, "BOTTOMLEFT", 0, 1)
        frame:SetFrameStrata("HIGH")

        frame:SetScript("OnShow", B.I.Reagent.Show)
        frame:SetScript("OnHide", B.I.Reagent.Hide)

        B.I.Backdrop(frame)
        T.F.mainframe.reagent = frame
    end

    local function create_characterframe()

        local frame = CreateFrame("Frame", "BWcharacter", T.F.mainframe)
        frame:SetPoint("TOPLEFT", T.F.mainframe, "BOTTOMLEFT", 0, 1)
        frame:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        frame:SetFrameStrata("HIGH")

        frame:SetScript("OnShow", B.I.Character.Show)
        frame:SetScript("OnHide", B.I.Character.Hide)

        B.I.Backdrop(frame)
        T.F.mainframe.character = frame
    end

    local function create_voidframe()

        local frame = CreateFrame("Frame", "BWvoid", T.F.mainframe)
        frame:SetPoint("TOPLEFT", T.F.mainframe, "BOTTOMLEFT", 0, 1)
        frame:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        frame:SetFrameStrata("HIGH")
        frame.lasttab = 1

        frame:SetScript("OnShow", B.I.Void.Show)
        frame:SetScript("OnHide", B.I.Void.Hide)

        B.I.Backdrop(frame)
        T.F.mainframe.void = frame
    end

    local function create_guildframe()

        local frame = CreateFrame("Frame", "BWguild", T.F.mainframe)
        frame:SetPoint("TOPLEFT", T.F.mainframe, "BOTTOMLEFT", 0, 1)
        frame:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        frame:SetFrameStrata("HIGH")
        frame.lasttab = 1

        frame:SetScript("OnShow", B.I.Guild.Show)
        frame:SetScript("OnHide", B.I.Guild.Hide)

        B.I.Backdrop(frame)
        T.F.mainframe.guild = frame
    end

    local function create_searchframe()

        local frame = CreateFrame("Frame", "BWsearch", T.F.mainframe)
        frame:SetHeight(200)
        frame:EnableMouse(true)

        T.F.mainframe.search = frame
    end

--[[//////////////////////////////////////////////////

    MAIN FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    function B.CreateFrames()

        create_mainframe()

        create_bankframe()
        create_reagentframe()
        create_characterframe()
        create_voidframe()
        create_guildframe()

        create_searchframe()
    end

    function B.CreateLayouts()

        create_bank()
        create_reagent()
        create_character()
        create_void()
        create_guild()

        create_search()

        if not T.CONFIG['bankbagstoggle'] then T.F.mainframe.bank.bags:Hide() end
        if not T.CONFIG['playerbagstoggle'] then T.F.mainframe.character.bags:Hide() end
    end

--[[//////////////////////////////////////////////////

    MAINFRAME FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    function B.ValidateBags(slots, parent, location, splitstack)

        local rows, num = location.rows, location.num
        local numslots = (T.ROW * (rows - 1)) + num

        if not slots or slots == numslots then
            return-- Already correct amount of slots
        elseif slots < numslots then -- Hide extra slots
            rows, num = B.I.Grid.Remove(parent, slots, rows, num)
        else -- Add or Show missing slots
            rows, num = B.I.Grid.Add(parent, slots, rows, num, splitstack)
        end

        location.rows, location.num = rows, num
        parent:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        parent:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.ValidateBankBags(bank)
        local total, page

        total = 0
        for page = 1, T.NUM_BAG_SLOTS + T.NUM_BANKBAGSLOTS do
            total = total + (bank[page] and bank[page]['slots'] or 0)
        end

        B.ValidateBags(total, T.F.mainframe.bank, B.I.bank, B.I.Bank.SplitStack)
    end

    function B.ValidatePlayerBags(bank)
        local total, page

        total = 0
        for page = 0, T.NUM_BAG_SLOTS do
            total = total + (bank[page] and bank[page]['slots'] or 0)
        end

        B.ValidateBags(total, T.F.mainframe.character, B.I.character, B.I.Character.SplitStack)
    end

    function B.I.Main.Frame.Button(label, parent, name)
        local button = CreateFrame("Button", name, parent or T.F.mainframe)

        button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        button:SetWidth(30)
        button:SetHeight(T.F.mainframe:GetHeight())
        button:SetNormalFontObject("GameFontNormal")
        button:SetText(label)
        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8",  edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1})
        button:SetBackdropBorderColor(0, 0, 0, 0)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")
        highlight:SetTexture("Interface\\Buttons\\WHITE8x8")
        highlight:SetHeight(2)
        highlight:SetVertexColor(1, 1, 1, 1)
        button:SetHighlightTexture(highlight)
        B.I.Main.Frame.ButtonHighlight(button, "BOTTOM")

        return button
    end

    function B.I.Main.Frame.OptionsButton(text, parent, name, inherit)

        local button = CreateFrame("Button", name, parent or T.F.mainframe, inherit)
        button:SetHeight(12)
        button:SetWidth(50)

        B.I.Backdrop(button)

        local label = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
        label:SetTextColor(1, 1, 1)
        label:SetAllPoints()
        label:SetFont(label:GetFont(), 7)
        label:SetJustifyH("CENTER")
        label:SetJustifyV("CENTER")
        label:SetWordWrap(false)
        label:SetText(text)
        button.label = label

        button:HookScript("OnEnter", function(self) self:SetBackdropBorderColor(1, 1, 1, 1) end)
        button:HookScript("OnLeave", function(self) self:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1) end)

        return button
    end

    local function create_framebutton(label, frame, scripts)

        local frame = B.I.Main.Frame.OptionsButton(label, frame)
        frame:HookScript("OnClick", scripts.Click)
        frame:HookScript("OnEnter", scripts.Enter)
        frame:HookScript("OnLeave", scripts.Leave)

        return frame
    end

    local function create_scroll(parent, name, update, scrollname, button, buttonheight)

        local list = CreateFrame("Frame", name, parent)
        list:SetPoint("TOPLEFT", parent, "TOPLEFT")
        list:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT")
        list:SetWidth(parent:GetWidth())

        parent.list = list
        parent.scroll = B.I.ScrollFrame(list, scrollname, update, button, buttonheight)
    end

--[[//////////////////////////////////////////////////

    OPTIONS FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    function B.I.Options:Show()
        T.F.status:SetParent(T.F.mainframe.optionbutton)
        T.F.status:SetVertexColor(T.CLASS.r, T.CLASS.g, T.CLASS.b, 1)
        B.SetFrameLocation()
    end

    function B.I.Options:Enter()

        T.F.status:SetVertexColor(1, 1, 1, 1)
        GameTooltip:ClearLines()
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:SetText(T.ACTIVE_TABLE.player.." ("..string.sub(T.ACTIVE_TABLE.faction, 1, 1)..")")
        GameTooltip:AddLine(GetMoneyString(B.GetPlayerBank('currency')['money'] or 0, true))
        GameTooltip:AddLine(" ")
        GameTooltip:AddLine(L[10005], 1, 1, 1) -- drag
        GameTooltip:AddLine(L[10006], 1, 1, 1) -- hide
        GameTooltip:AddLine(L[10007], 1, 1, 1) -- menu
        GameTooltip:Show()
    end

    function B.I.Options:Click(button)

        if button == "RightButton" then
            CloseDropDownMenus()
            T.F.mainframe.togglebutton:Show()
        else
            if _G["DropDownList1"]:IsShown() then
                CloseDropDownMenus()
            else
                ToggleDropDownMenu(1, nil, _G['BWdropDownMenu'], "BWoptionbutton", 0, 0, nil, nil, 2)
            end
        end 
    end

    function B.I.Options:DragStart()
        T.F.status:SetVertexColor(0.18, 0.64, 1, 1)
        T.F.mainframe:StartMoving()
    end

    function B.I.Options:DragStop()
        T.F.status:SetVertexColor(T.CLASS.r, T.CLASS.g, T.CLASS.b, 1)
        T.F.mainframe:StopMovingOrSizing()
        B.SetFrameLocation()
    end

    function B.I.Options:Leave()
        T.F.status:SetVertexColor(T.CLASS.r, T.CLASS.g, T.CLASS.b, 1)
        GameTooltip:Hide()
    end

    function B.I.Toggle:Show()
        T.F.status:SetParent(T.F.mainframe.togglebutton)
        T.F.status:SetVertexColor(0.37, 0.07, 0.02, 1)
        T.F.mainframe:Hide()
    end

    function B.I.Toggle:Hide()
        T.F.mainframe:Show()
    end

    function B.I.Toggle:Click()
        self:Hide()
    end

    function B.I.Toggle:Enter()
        T.F.status:SetVertexColor(0.01, 0.79, 0.15, 1)
        self:SetBackdropBorderColor(0.01, 0.79, 0.15, 1)
    end

    function B.I.Toggle:Leave()
        T.F.status:SetVertexColor(0.37, 0.07, 0.02, 1)
        self:SetBackdropBorderColor(0.15, 0.15, 0.15, 1)
    end

    function B.I.OptionsDropDown.Create()

        local menu = CreateFrame("Frame", "BWdropDownMenu", T.F.mainframe, "UIDropDownMenuTemplate")
        menu.point = "TOPLEFT"
        menu.relativePoint = "BOTTOMLEFT"
        menu:Hide()

        UIDropDownMenu_SetWidth(menu, 130, 4)
        UIDropDownMenu_Initialize(menu, B.I.OptionsDropDown.Initialize) --The virtual
    end

    function B.I.OptionsDropDown:Initialize(level)
        --drop down menues can have sub menues. The value of level determines the drop down sub menu tier
        local UIDROPDOWNMENU_MENU_VALUE = UIDROPDOWNMENU_MENU_VALUE
        local BWsavedtable = BWsavedtable
        local level = level or 1
        local x, k, v, class, checked, faction, player

        if level == 1 then
            local info = UIDropDownMenu_CreateInfo()

            if BWsavedtable then
                for k, v in pairs(BWsavedtable) do
                    if not T.OPTIONS[k] then
                        B.I.OptionsDropDown.Menu(info, k, k, level)
                    end
                end
            end

            B.I.OptionsDropDown.Empty(level)

            if T.ACTIVE_TABLE.player then
                B.I.OptionsDropDown.Button(self, info, L[10011].." "..T.ACTIVE_TABLE.player, "delete", false, level)
            end

            B.I.OptionsDropDown.Empty(level)

            if T.CONFIG then
                --- Enable search all servers
                B.I.OptionsDropDown.Button(self, info, L[10059], "searchallservers", BWsavedtable['enablesearchallservers'], level)
                --- Enable search opposite faction
                B.I.OptionsDropDown.Button(self, info, L[10060], "searchallfactions", BWsavedtable['enablesearchallfactions'], level)
            end

            --- Money gain tracker
            B.I.OptionsDropDown.Menu(info, L[10038], "moneytracker", level)

        elseif level == 2 and UIDROPDOWNMENU_MENU_VALUE == "moneytracker" then
            local info = UIDropDownMenu_CreateInfo()
            --- sounds
            B.I.OptionsDropDown.Menu(info, L[10040], "soundsmoneytracker", level)
            --- Money stack
            B.I.OptionsDropDown.Menu(info, L[10041], "limitmoneytracker", level)
            --- Move tracker
            B.I.OptionsDropDown.Button(self, info, L[10042], "movemoneytracker", false, level)
            --- Enable sounds
            B.I.OptionsDropDown.Button(self, info, L[10047], "enablesoundsmoneytracker", T.CONFIG['enablesoundsmoneytracker'], level)
            --- Enable tracker
            B.I.OptionsDropDown.Button(self, info, L[10039], "enablemoneytracker", T.CONFIG['enablemoneytracker'], level)

        elseif level == 2 then -- server
            local info = UIDropDownMenu_CreateInfo()

            for k, v in pairs(BWsavedtable[UIDROPDOWNMENU_MENU_VALUE]) do
                checked = tostring(T.ACTIVE_TABLE.player.."."..string.sub(T.ACTIVE_TABLE.faction, 1, 1)) == k and true or false
                player, faction = k:match"([^.]*).(.*)"
                class = "|c"..(v['player']['class'] and T.RAID_CLASS_COLORS[v['player']['class']].colorStr or "ffffffff")
                B.I.OptionsDropDown.Button(self, info, player.." ("..faction..")", UIDROPDOWNMENU_MENU_VALUE.."."..k, checked, level, class)
            end

        elseif level == 3 and UIDROPDOWNMENU_MENU_VALUE == "soundsmoneytracker" then
            local info = UIDropDownMenu_CreateInfo()

            --- Money
            B.I.OptionsDropDown.Menu(info, L[10043], "moneysound", level)
            --- Treasure
            B.I.OptionsDropDown.Menu(info, L[10044], "treasuresound", level)

        elseif level == 3 and UIDROPDOWNMENU_MENU_VALUE == "limitmoneytracker" then
            local info = UIDropDownMenu_CreateInfo()

            --- Set minimum
            B.I.OptionsDropDown.Button(self, info, L[10050], "minimummoneytracker", false, level)
            --- Set extra money value
            B.I.OptionsDropDown.Button(self, info, L[10051], "treasuremoneytracker", false, level)

        elseif level == 4 and UIDROPDOWNMENU_MENU_VALUE == "moneysound" then
            local info = UIDropDownMenu_CreateInfo()

            --- Coins
            B.I.OptionsDropDown.Button(self, info, L[10045], "coinssound", T.CONFIG['moneysound'] == 1 and true or false, level)
            --- Retro coins
            B.I.OptionsDropDown.Button(self, info, L[10046], "retrocoinssound", T.CONFIG['moneysound'] == 2 and true or false, level)

        elseif level == 4 and UIDROPDOWNMENU_MENU_VALUE == "treasuresound" then
            local info = UIDropDownMenu_CreateInfo()

            --- Treasure 1
            B.I.OptionsDropDown.Button(self, info, L[10048], "treasure1sound", T.CONFIG['treasuresound'] == 3 and true or false, level)
            --- Treasure 2
            B.I.OptionsDropDown.Button(self, info, L[10049], "treasure2sound", T.CONFIG['treasuresound'] == 4 and true or false, level)

        end
    end

    function B.I.OptionsDropDown.Menu(info, label, value, level)

        info.text = label
        info.value = value
        info.hasArrow = true
        info.checked = false
        info.keepShownOnClick = true

        UIDropDownMenu_AddButton(info, level)
    end

    function B.I.OptionsDropDown.Button(self, info, label, value, checked, level, hex, icon)

        info.text = label
        info.hasArrow = false
        info.value = value
        info.func = B.dropDownMenuItem_OnClick
        info.owner = self:GetParent()
        info.checked = checked
        info.colorCode = hex

        UIDropDownMenu_AddButton(info, level)
    end

    function B.I.OptionsDropDown.Empty(level)

        local info = {} --UIDropDownMenu_CreateInfo() For some reason, it messes with any info called beyond that...
        info.disabled = true
        info.notCheckable = true
        info.notClickable = true

        UIDropDownMenu_AddButton(info, level)
    end

--[[//////////////////////////////////////////////////

    BANK FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    local create_bankbags, create_bankbuttons

    function create_bank()

        T.F.mainframe.bank.tooltiptoggle = create_framebutton(L[10058], T.F.mainframe.bank, B.I.Bank.Tooltip)
        T.F.mainframe.bank.bagstoggle = create_framebutton(L[10057], T.F.mainframe.bank, B.I.Bank.BagToggle)

        create_bankbags()
        create_bankbuttons()
    end

    function create_bankbags()
        local x, lasttab

        T.F.mainframe.bank.bags = CreateFrame("Frame", nil, T.F.mainframe.bank)
        for x = 1, T.NUM_BANKBAGSLOTS do

            T.F.mainframe.bank.bags[x] = {}
            T.F.mainframe.bank.bags[x].tab = B.I.Tab.Create(T.F.mainframe.bank.bags, "BWbank"..x)
            T.F.mainframe.bank.bags[x].tab.search:SetAllPoints(T.F.mainframe.bank.bags[x].tab)
            T.F.mainframe.bank.bags[x].tab.index = x

            T.F.mainframe.bank.bags[x].tab:SetScript("OnEnter", B.I.Bank.Bags.Enter)
            T.F.mainframe.bank.bags[x].tab:SetScript("OnLeave", B.I.Bank.Bags.Leave)

            if x == 1 then
                T.F.mainframe.bank.bags[x].tab:SetPoint("BOTTOMLEFT", T.F.mainframe.bank, "TOPLEFT")
            else
                T.F.mainframe.bank.bags[x].tab:SetPoint("TOPLEFT", lasttab, "TOPRIGHT", 2, 0)
            end
            lasttab = T.F.mainframe.bank.bags[x].tab
        end
    end

    function create_bankbuttons()

        B.I.Frame.Create(T.F.mainframe.bank, T.F.mainframe.bank)
        B.I.bank.rows, B.I.bank.num = B.I.Grid.Create(T.F.mainframe.bank, T.NUM_BANKGENERIC_SLOTS, B.I.bank.rows, B.I.bank.num, B.I.Bank.SplitStack)

        T.F.mainframe.bank:SetHeight(10 + (T.BUTTON_HEIGHT * (B.I.bank.rows + 1)) + (T.FRAMEWHITESPACE * B.I.bank.rows))
        T.F.mainframe.bank:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.I.Bank.SplitStack(button, split)
        local bag, slot = B.GetBankBag(button)
        SplitContainerItem(bag, slot, split)
    end

    function B.I.Bank:Click(...)
        local frame = self:GetParent()

        if not T.ISBANKOPEN then
            frame.character:Hide()
        elseif T.ISBANKOPEN and frame.character:IsVisible() then
            -- move bags
            B.AnchorBags(frame.bank)
        end

        frame.reagent:Hide()
        frame.void:Hide()
        frame.guild:Hide()
        frame.search:Hide()

        if frame.bank:IsVisible() then
            frame.lastbank = nil
            frame.bank:Hide()
        else
            frame.lastbank = "bank"
            frame.bank:Show()
        end
    end

    function B.I.Bank.BagToggle:Click(...)
        if T.F.mainframe.bank.bags:IsVisible() then
            T.F.mainframe.bank.bags:Hide()
            T.CONFIG['bankbagstoggle'] = false
        else
            T.F.mainframe.bank.bags:Show()
            T.CONFIG['bankbagstoggle'] = true
        end
    end

    function B.I.Bank.BagToggle:Enter()
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10008])
        GameTooltip:Show()
    end

    function B.I.Bank.BagToggle:Leave()
        GameTooltip:Hide()
    end

    function B.I.Bank.Tooltip:Click(...)
        T.CONFIG['bankbagtooltiptoggle'] = not T.CONFIG['bankbagtooltiptoggle']
    end

    function B.I.Bank.Tooltip:Enter()
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10009])
        GameTooltip:Show()
    end

    function B.I.Bank.Tooltip:Leave()
        GameTooltip:Hide()
    end

    function B.I.Bank.Bags:Enter(...)
        if (T.CONFIG and T.CONFIG['bankbagtooltiptoggle']) or not T.CONFIG then
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
            if self.link then GameTooltip:SetHyperlink(self.link) end
            if self.quality then
                local r, g, b = GetItemQualityColor(self.quality)
                GameTooltip:SetBackdropBorderColor(r, g, b)
            end
            GameTooltip:Show()
        end

        if #T.BW.items == 0 then
            B.BankBagSearch(self.index + 4)
        end
    end

    function B.I.Bank.Bags:Leave(...)
        GameTooltip:Hide()

        local x, y = B.GetCursorScaledPosition()
        -- Only hide when cursor exits left or right (with exceptions)
        if #T.BW.items == 0 then
            if self.index == 1 then -- First bag tab
                if (x < self:GetLeft()) or (y > self:GetTop()) or (y < self:GetBottom()) then
                    B.BankBagSearch(false)
                end
            elseif self.index == #T.F.mainframe.bank.bags then -- Last bag tab
                if (x > self:GetLeft()) or (y > self:GetTop()) or (y < self:GetBottom()) then
                    B.BankBagSearch(false)
                end
            elseif (y > self:GetTop()) or (y < self:GetBottom()) then -- Middle bags
                B.BankBagSearch(false)
            end
        end
    end

--[[//////////////////////////////////////////////////

    REAGENT FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    local create_reagentbuttons

    function create_reagent()
        create_reagentbuttons()
    end

    function create_reagentbuttons()
        local x, rows

        for x = 1, T.REAGENT_PAGES do
            T.F.mainframe.reagent[x] = {}
            B.I.Frame.Create(T.F.mainframe.reagent, T.F.mainframe.reagent[x], x)
            rows = B.I.Grid.Create(T.F.mainframe.reagent[x], T.NUM_REAGENT_SLOTS, nil, nil, B.I.Reagent.SplitStack)
        end

        T.F.mainframe.reagent:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.reagent:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.I.Reagent:Click(...)

        local frame = self:GetParent()

        if not T.ISBANKOPEN then
            frame.character:Hide()
        elseif T.ISBANKOPEN and frame.character:IsVisible() then
            -- move bags
            B.AnchorBags(frame.bank)
        end

        frame.bank:Hide()
        frame.void:Hide()
        frame.guild:Hide()
        frame.search:Hide()

        if frame.reagent:IsVisible() then
            frame.lastbank = nil
            frame.reagent:Hide()
        else
            frame.lastbank = "reagent"
            frame.reagent:Show()
        end
    end

    function B.I.Reagent:Show()
        T.F.mainframe.reagentbutton:LockHighlight()
    end

    function B.I.Reagent:Hide()
        T.F.mainframe.reagentbutton:UnlockHighlight()
    end

    function B.I.Reagent.SplitStack(button, split)
        SplitContainerItem(T.REAGENTBANK_CONTAINER, button:GetID(), split)
    end

--[[//////////////////////////////////////////////////

    CHARACTER FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    local create_characterbags, create_characterbuttons, create_characterequipment, create_characterequipmentbuttons
    local create_charactertab, create_characterartifact, create_characterauctions, create_charactermail
    local create_auctionsbutton, create_mailbutton

    function create_character()

        T.F.mainframe.character.tooltiptoggle = create_framebutton(L[10058], T.F.mainframe.character, B.I.Character.Tooltip)
        T.F.mainframe.character.bagstoggle = create_framebutton(L[10057], T.F.mainframe.character, B.I.Character.BagToggle)

        create_charactertab()
        create_characterauctions()
        create_charactermail()
        create_characterartifact()
        create_characterbags()
        create_characterbuttons()
        create_characterequipmentbuttons()
    end

    function create_characterbags()
        local x, lasttab

        T.F.mainframe.character.bags = CreateFrame("Frame", nil, T.F.mainframe.character)
        for i = 1, T.NUM_BAG_SLOTS do
            T.F.mainframe.character.bags[i] = {}
            T.F.mainframe.character.bags[i].tab = B.I.Tab.Create(T.F.mainframe.character.bags, "BWcharacter"..i)
            T.F.mainframe.character.bags[i].tab.search:SetAllPoints(T.F.mainframe.character.bags[i].tab) -- since we have permanent borders
            T.F.mainframe.character.bags[i].tab.index = i

            T.F.mainframe.character.bags[i].tab:SetScript("OnEnter", B.I.Character.Bags.Enter)
            T.F.mainframe.character.bags[i].tab:SetScript("OnLeave", B.I.Character.Bags.Leave)

            if i == 1 then
                T.F.mainframe.character.bags[i].tab:SetPoint("BOTTOMLEFT", T.F.mainframe.character, "TOPLEFT")
            else
                T.F.mainframe.character.bags[i].tab:SetPoint("TOPLEFT", lasttab, "TOPRIGHT", 2, 0)
            end
            lasttab = T.F.mainframe.character.bags[i].tab
        end
    end

    function create_characterbuttons()

        B.I.Frame.Create(T.F.mainframe.character, T.F.mainframe.character)
        B.I.character.rows, B.I.character.num = B.I.Grid.Create(T.F.mainframe.character, 30, B.I.character.rows, B.I.character.num, B.I.Character.SplitStack)

        T.F.mainframe.character:SetHeight(10 + (T.BUTTON_HEIGHT * (B.I.character.rows + 1)) + (T.FRAMEWHITESPACE * B.I.character.rows))
        T.F.mainframe.character:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW)))
    end

    function B.I.Character.SplitStack(button, split)
        local bag, slot = B.GetBag(button)
        SplitContainerItem(bag, slot, split)
    end

    function B.I.Character:Show()
        T.F.mainframe.characterbutton:LockHighlight()
    end

    function B.I.Character:Hide()
        T.F.mainframe.characterbutton:UnlockHighlight()
    end

    function B.I.Character:Click(...)

        local frame = self:GetParent()

        frame.bank:Hide()
        frame.reagent:Hide()
        frame.void:Hide()
        frame.guild:Hide()
        frame.search:Hide()

        if frame.character:IsVisible() then
            frame.lastbank = nil
            frame.character:Hide()
        else
            frame.lastbank = "character"
            frame.character:Show()
        end
    end

    function B.I.Character.Bags:Enter(...)
        if (T.CONFIG and T.CONFIG['playerbagtooltiptoggle']) or not T.CONFIG then
            local r, g, b = GetItemQualityColor(self.quality)
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
            if self.link then GameTooltip:SetHyperlink(self.link) end
            GameTooltip:SetBackdropBorderColor(r, g, b)
            GameTooltip:Show()
        end

        if #T.BW.items == 0 then
            B.PlayerBagSearch(self.index)
        end
    end

    function B.I.Character.Bags:Leave(...)
        GameTooltip:Hide()

        local x, y = B.GetCursorScaledPosition()
        -- Only hide when cursor exits left or right (with exceptions)
        if #T.BW.items == 0 then
            if self.index == 1 then -- First bag tab
                if T.F.mainframe.layout > 2 then
                    if (x < self:GetLeft()) or (y > self:GetTop()) or (y < self:GetBottom()) then
                        B.PlayerBagSearch(false)
                    end
                else
                    if (x > self:GetLeft()) or (y > self:GetTop()) or (y < self:GetBottom()) then
                        B.PlayerBagSearch(false)
                    end
                end
            elseif self.index == #T.F.mainframe.character.bags then -- Last bag tab
                if T.F.mainframe.layout > 2 then
                    if (x > self:GetLeft()) or (y > self:GetTop()) or (y < self:GetBottom()) then
                        B.PlayerBagSearch(false)
                    end
                else
                    if (x < self:GetLeft()) or (y > self:GetTop()) or (y < self:GetBottom()) then
                        B.PlayerBagSearch(false)
                    end
                end
            elseif (y > self:GetTop()) or (y < self:GetBottom()) then -- Middle bags
                B.PlayerBagSearch(false)
            end
        end
    end

    function create_charactertab()

        T.F.mainframe.character.tab = B.I.Tab.Create(T.F.mainframe.character)
        T.F.mainframe.character.tab:SetPoint("TOPLEFT", T.F.mainframe.character, "TOPRIGHT")
        T.F.mainframe.character.tab:SetNormalTexture("Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs")
        T.F.mainframe.character.tab:GetNormalTexture():SetTexCoord(0.01562500, 0.53125000, 0.46875000, 0.60546875)
        T.F.mainframe.character.tab:SetBackdropColor(0.05, 0.05, 0.05, 1)

        T.F.mainframe.character.tab:HookScript("OnClick", B.I.Character.Tab.Click)
        T.F.mainframe.character.tab:HookScript("OnEnter", B.I.Character.Tab.Enter)
        T.F.mainframe.character.tab:SetScript("OnLeave", function(self) GameTooltip:Hide() end)

        create_characterequipment()
    end

    function B.I.Character.Tab:Click(...)
        T.F.mainframe.character.auctions:Hide()
        T.F.mainframe.character.mail:Hide()

        if T.F.mainframe.character.equipment:IsVisible() then
            self:SetBackdropBorderColor(0, 0, 0, 1)
            T.F.mainframe.character.equipment:Hide()
        else
            self:SetBackdropBorderColor(1, 1, 1, 1)
            T.F.mainframe.character.equipment:Show()
        end
    end

    function B.I.Character.Tab:Enter(...)
        if T.F.mainframe.layout > 2 then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT", 10, -10)
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        end
        GameTooltip:SetText(L[9011])
        GameTooltip:Show()
    end

    function B.I.Character.Tooltip:Click(...)
        T.CONFIG['playerbagtooltiptoggle'] = not T.CONFIG['playerbagtooltiptoggle']
    end

    function B.I.Character.Tooltip:Enter()
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10009])
        GameTooltip:Show()
    end

    function B.I.Character.Tooltip:Leave()
        GameTooltip:Hide()
    end

    function B.I.Character.BagToggle:Click(...)

        if T.F.mainframe.character.bags:IsVisible() then
            T.F.mainframe.character.bags:Hide()
            T.CONFIG['playerbagstoggle'] = false
        else
            T.F.mainframe.character.bags:Show()
            T.CONFIG['playerbagstoggle'] = true
        end
    end

    function B.I.Character.BagToggle:Enter()
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10008])
        GameTooltip:Show()
    end

    function B.I.Character.BagToggle:Leave()
        GameTooltip:Hide()
    end

    function create_characterartifact()

        T.F.mainframe.character.artifact = B.I.Tab.Create(T.F.mainframe.character, "BWartifact", "SecureActionButtonTemplate")
        T.F.mainframe.character.artifact:SetPoint("TOPLEFT", T.F.mainframe.character.mail.tab, "BOTTOMLEFT", 0, -2)
        T.F.mainframe.character.artifact:SetNormalTexture(463517)
        T.F.mainframe.character.artifact:SetBackdropColor(0.05, 0.05, 0.05, 1)
        T.F.mainframe.character.artifact:SetAttribute("type", "macro")
        T.F.mainframe.character.artifact:Show() -- To make keybinding work without first displaying

        T.F.mainframe.character.artifact:HookScript("PreClick", B.I.Character.Artifact.PreClick)
        T.F.mainframe.character.artifact:HookScript("OnClick", B.I.Character.Artifact.Click)
        T.F.mainframe.character.artifact:HookScript("OnEnter", B.I.Character.Artifact.Enter)
        T.F.mainframe.character.artifact:HookScript("OnLeave", B.I.Character.Artifact.Leave)
    end

    function B.I.Character.Artifact:PreClick(...)
        local x, page, counter, button, buttons, name
        local player = T['player']

        local counter = 1
        for page = 0, #player do
            if player[page] then
                buttons = T.F.mainframe.character.buttons
                for x = 1, player[page]['slots'] do
                    button = buttons[counter]

                    if button.name then
                        if IsArtifactPowerItem(B.GetItemIdFromLink(button.link)) then
                            self:SetAttribute("macrotext", format("/use %d %d", B.GetBag(button)))
                            B.print("Using: |T"..button.icon:GetTexture()..":0|t"..button.link..": "..B.DetectAP(B.GetBag(button)).." on "..player[-1][16]['link'], 1)
                            return
                        end
                    end
                    counter = counter + 1
                end
            end
        end
    end

    function B.I.Character.Artifact:Click(...)
        self:SetAttribute("macrotext", nil)
    end

    function B.I.Character.Artifact:Enter()
        self.total = nil
        B.PlayerArtifactSearch()

        if T.F.mainframe.layout > 2 then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT", 10, -10)
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        end
        GameTooltip:SetText(T.ITEM_QUALITY_COLORS[6].hex..L[9010].."|r") -- "|cffBD76FF"
        GameTooltip:AddLine(L[10019])
        if self.total then
            GameTooltip:AddLine(" ")
            GameTooltip:AddLine("|cffffffff"..self.total.."|r")
        end
        GameTooltip:Show()
    end

    function B.I.Character.Artifact:Leave()

        local text = T.F.mainframe.searchbar:GetText()
        if text and text ~= "" then
            B.PlayerSearch(text)
        else
            B.PlayerSearch(false)
        end

        GameTooltip:Hide()
    end

    function create_characterequipment()

        T.F.mainframe.character.equipment = CreateFrame("Frame", "BWcharacterequipment", T.F.mainframe.character)
        T.F.mainframe.character.equipment:SetPoint("TOPLEFT", T.F.mainframe.character.tab, "TOPRIGHT")
        T.F.mainframe.character.equipment:SetWidth(20 + (T.BUTTON_HEIGHT * 5) + (T.FRAMEWHITESPACE * (5 - 1)))
        T.F.mainframe.character.equipment:SetFrameStrata("HIGH")
        T.F.mainframe.character.equipment:Hide()

        T.F.mainframe.character.equipment:SetScript("OnShow", function(self)
            self:GetParent().tab:SetBackdropBorderColor(1, 1, 1, 1)
        end)
        T.F.mainframe.character.equipment:SetScript("OnHide", function(self)
            self:GetParent().tab:SetBackdropBorderColor(0, 0, 0, 1)
        end)

        B.I.Backdrop(T.F.mainframe.character.equipment)
    end

    function create_characterequipmentbuttons()
        local x, button, lastbutton, lastrow, rows, num

        B.I.Frame.Create(T.F.mainframe.character.equipment, T.F.mainframe.character.equipment)
        rows, num = 0, 0

        for x = 1, T.EQUIPMENT_SLOTS do

            button = B.I.Button.Create(x, T.F.mainframe.character.equipment.frame)
            button:SetScript("OnEnter", B.I.Button.Enter)
            button:SetScript("OnLeave", B.I.Button.Leave)
            button:HookScript("OnEnter", function(self)
                if not self.link then
                    GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
                    GameTooltip:SetText(_G[T.INVSLOTS[self:GetID()]])
                    GameTooltip:Show()
                end
            end)
            
            if x == 1 then -- First slot
                button:SetPoint("TOPLEFT")
                rows, num = rows + 1, 1
                lastrow = button

            elseif num == 5 then -- Last column of the row
                button:SetPoint("TOP", lastrow, "BOTTOM", 0, -T.FRAMEWHITESPACE)
                rows, num = rows + 1, 1
                lastrow = button
            else
                button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", T.FRAMEWHITESPACE, 0)
                num = num + 1
            end
            T.F.mainframe.character.equipment.buttons[x] = button
            lastbutton = button
        end

        T.F.mainframe.character.equipment:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.character.equipment:SetWidth(20 + (T.BUTTON_HEIGHT * 5) + (T.FRAMEWHITESPACE * (5 - 1)))
    end

    function create_characterauctions()

        T.F.mainframe.character.auctions = CreateFrame("Frame", "BWcharacterauctions", T.F.mainframe.character)
        T.F.mainframe.character.auctions:SetPoint("TOPLEFT", T.F.mainframe.character.tab, "TOPRIGHT")
        T.F.mainframe.character.auctions:SetWidth(220)
        T.F.mainframe.character.auctions:SetHeight(250)
        T.F.mainframe.character.auctions:SetFrameStrata("HIGH")
        T.F.mainframe.character.auctions.total = 0
        T.F.mainframe.character.auctions.totalitems = 0

        T.F.mainframe.character.auctions:Hide()
        T.F.mainframe.character.auctions:EnableMouse(true)

        T.F.mainframe.character.auctions:SetScript("OnShow", function(self)
            self.tab:SetBackdropBorderColor(1, 1, 1, 1)
        end)
        T.F.mainframe.character.auctions:SetScript("OnHide", function(self)
            self.tab:SetBackdropBorderColor(0, 0, 0, 1)
        end)

        T.F.mainframe.character.auctions.enddot = T.F.mainframe.character.auctions:CreateTexture(nil, "OVERLAY")
        T.F.mainframe.character.auctions.enddot:SetMask("Textures\\MinimapMask")
        T.F.mainframe.character.auctions.enddot:SetHeight(5)
        T.F.mainframe.character.auctions.enddot:SetWidth(5)
        T.F.mainframe.character.auctions.enddot:SetTexture("Interface\\Buttons\\WHITE8x8")
        T.F.mainframe.character.auctions.enddot:SetVertexColor(1, 1, 1, 1)
        T.F.mainframe.character.auctions.enddot:SetPoint("TOP", T.F.mainframe.character.auctions, "BOTTOM", 0, 12)

        T.F.mainframe.character.auctions.tab = B.I.Tab.Create(T.F.mainframe.character)
        T.F.mainframe.character.auctions.tab:SetPoint("TOPLEFT", T.F.mainframe.character.tab, "BOTTOMLEFT", 0, -2)
        T.F.mainframe.character.auctions.tab:SetNormalTexture("Interface\\Icons\\INV_Misc_Coin_04")

        T.F.mainframe.character.auctions.tab:HookScript("OnClick", B.I.Character.Auctions.Click)
        T.F.mainframe.character.auctions.tab:HookScript("OnEnter", B.I.Character.Auctions.Enter)
        T.F.mainframe.character.auctions.tab:SetScript("OnLeave", B.I.Character.Auctions.Leave)

        create_scroll(T.F.mainframe.character.auctions, "FauxScrollFrameBWauctions", B.ScrollbarUpdateAuctions, "BWauctionsScrollFrame", create_auctionsbutton, 28)
    end

    function create_auctionsbutton(index)

        local button = B.I.TabList.Button(index, T.F.mainframe.character.auctions.list)

        button:SetScript("OnEnter", B.I.Character.Auctions.Button.Enter)
        button:SetScript("OnLeave", B.I.Character.Auctions.Button.Leave)
        button:SetScript("OnClick", B.I.Character.Auctions.Button.Click)

        button.leftstring:SetPoint("LEFT", button.icon, "RIGHT", 10, 0)
        button.leftstring:SetPoint("RIGHT", button, "RIGHT", -button.rightstring.bg:GetWidth(), 0)

        return button
    end

    function B.I.Character.Auctions:Click(...)

        T.F.mainframe.character.equipment:Hide()
        T.F.mainframe.character.mail:Hide()

        if T.F.mainframe.character.auctions:IsVisible() then
            T.F.mainframe.character.auctions:Hide()
        else
            B.ScrollbarUpdateAuctions()
            T.F.mainframe.character.auctions:Show()
        end
    end

    function B.I.Character.Auctions:Enter()

        if T.F.mainframe.layout > 2 then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT", 10, -10)
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        end

        local text = L[9001]
        if T.F.mainframe.character.auctions.totalitems > 0 then
            text = text.." ("..T.F.mainframe.character.auctions.totalitems..")"
        end
        GameTooltip:SetText(text)

        if T.F.mainframe.character.auctions.total > 0 then
            GameTooltip:AddLine(GetMoneyString(T.F.mainframe.character.auctions.total, true))
        end
        GameTooltip:Show()
    end

    function B.I.Character.Auctions:Leave()
        GameTooltip:Hide()
    end

    function B.I.Character.Auctions.Button:Enter()

        local r, g, b = GetItemQualityColor(self.quality)
        self:SetBackdropBorderColor(r, g, b, 1)

        B.I.Button.Enter(self, false)
    end

    function B.I.Character.Auctions.Button:Leave()
        self:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        GameTooltip:Hide()
    end

    function B.I.Character.Auctions.Button:Click()
        T.F.mainframe.searchbar:SetText(self.name)
    end

    function create_charactermail()

        T.F.mainframe.character.mail = CreateFrame("Frame", "BWcharactermail", T.F.mainframe.character)
        T.F.mainframe.character.mail:SetPoint("TOPLEFT", T.F.mainframe.character.tab, "TOPRIGHT")
        T.F.mainframe.character.mail:SetWidth(220)
        T.F.mainframe.character.mail:SetHeight(250)
        T.F.mainframe.character.mail:SetFrameStrata("HIGH")
        T.F.mainframe.character.mail.totalitems = 0
        T.F.mainframe.character.mail.money = 0
        T.F.mainframe.character.mail.cod = 0

        T.F.mainframe.character.mail:Hide()
        T.F.mainframe.character.mail:EnableMouse(true)

        T.F.mainframe.character.mail:SetScript("OnShow", function(self)
            self.tab:SetBackdropBorderColor(1, 1, 1, 1)
        end)
        T.F.mainframe.character.mail:SetScript("OnHide", function(self)
            self.tab:SetBackdropBorderColor(0, 0, 0, 1)
        end)

        T.F.mainframe.character.mail.enddot = T.F.mainframe.character.mail:CreateTexture(nil, "OVERLAY")
        T.F.mainframe.character.mail.enddot:SetMask("Textures\\MinimapMask")
        T.F.mainframe.character.mail.enddot:SetHeight(5)
        T.F.mainframe.character.mail.enddot:SetWidth(5)
        T.F.mainframe.character.mail.enddot:SetTexture("Interface\\Buttons\\WHITE8x8")
        T.F.mainframe.character.mail.enddot:SetVertexColor(1, 1, 1, 1)
        T.F.mainframe.character.mail.enddot:SetPoint("TOP", T.F.mainframe.character.mail, "BOTTOM", 0, 12)

        T.F.mainframe.character.mail.tab = B.I.Tab.Create(T.F.mainframe.character)
        T.F.mainframe.character.mail.tab:SetPoint("TOPLEFT", T.F.mainframe.character.auctions.tab, "BOTTOMLEFT", 0, -2)
        T.F.mainframe.character.mail.tab:SetNormalTexture(B.GetMailIcon())

        T.F.mainframe.character.mail.tab:HookScript("OnClick", B.I.Character.Mail.Click)
        T.F.mainframe.character.mail.tab:HookScript("OnEnter", B.I.Character.Mail.Enter)
        T.F.mainframe.character.mail.tab:SetScript("OnLeave", B.I.Character.Mail.Leave)

        create_scroll(T.F.mainframe.character.mail, "FauxScrollFrameBWmail", B.ScrollbarUpdateMail, "BWmailScrollFrame", create_mailbutton, 28)
    end

    function create_mailbutton(index)

        local button = B.I.TabList.Button(index, T.F.mainframe.character.mail.list)

        button:SetScript("OnEnter", B.I.Character.Mail.Button.Enter)
        button:SetScript("OnLeave", B.I.Character.Mail.Button.Leave)
        button:SetScript("OnClick", B.I.Character.Mail.Button.Click)

        button.leftstring:SetPoint("LEFT", button.icon, "RIGHT", 10, 0)
        button.leftstring:SetPoint("RIGHT", button, "RIGHT", -button.rightstring.bg:GetWidth(), 0)

        button.rightstring.time:Hide()

        return button
    end

    function B.I.Character.Mail:Click(...)

        T.F.mainframe.character.equipment:Hide()
        T.F.mainframe.character.auctions:Hide()

        if T.F.mainframe.character.mail:IsVisible() then
            T.F.mainframe.character.mail:Hide()
        else
            B.ScrollbarUpdateMail()
            T.F.mainframe.character.mail:Show()
        end
    end

    function B.I.Character.Mail:Enter()

        if T.F.mainframe.layout > 2 then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT", 10, -10)
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        end

        local text = L[9012]

        if T.F.mainframe.character.mail.totalitems == 50 then
            text = text.." (~"..T.F.mainframe.character.mail.totalitems.."*)"
        elseif T.F.mainframe.character.mail.totalitems > 0 then
            text = text.." ("..T.F.mainframe.character.mail.totalitems..")"
        end
        GameTooltip:SetText(text)

        if T.F.mainframe.character.mail.money > 0 then
            GameTooltip:AddLine(L[9008].." "..GetMoneyString(T.F.mainframe.character.mail.money, true))
        end
        if T.F.mainframe.character.mail.cod > 0 then
            GameTooltip:AddLine(L[9009].." "..GetMoneyString(T.F.mainframe.character.mail.cod, true))
        end
        GameTooltip:Show()
    end

    function B.I.Character.Mail:Leave()
        GameTooltip:Hide()
    end

    function B.I.Character.Mail.Button:Enter()

        local r, g, b = GetItemQualityColor(self.quality)
        self:SetBackdropBorderColor(r, g, b, 1)

        B.I.Button.Enter(self, false)
    end

    function B.I.Character.Mail.Button:Leave()
        self:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        GameTooltip:Hide()
    end

    function B.I.Character.Mail.Button:Click()
        T.F.mainframe.searchbar:SetText(self.name)
    end

--[[//////////////////////////////////////////////////

    VOID FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    local create_voidbuttons, create_voidtabs

    function create_void()

        create_voidtabs()
        create_voidbuttons()
    end

    function create_voidtabs()
        local x, lasttab

        for x = 1, T.VOID_PAGES do
            T.F.mainframe.void[x] = {}
            T.F.mainframe.void[x].tab = B.I.Tab.Create(T.F.mainframe.void)
            T.F.mainframe.void[x].tab.index = x
            T.F.mainframe.void[x].tab:SetScript("OnClick", B.I.Void.Tabs.Click)

            if x == 1 then
                T.F.mainframe.void[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
                T.F.mainframe.void[x].tab:SetNormalTexture("Interface\\Icons\\INV_Enchant_EssenceCosmicGreater")
                T.F.mainframe.void[x].tab:SetPoint("TOPLEFT", T.F.mainframe.void, "TOPRIGHT")
            else
                T.F.mainframe.void[x].tab:SetNormalTexture("Interface\\Icons\\INV_Enchant_EssenceArcaneLarge")
                T.F.mainframe.void[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = T.F.mainframe.void[x].tab
        end
    end

    function create_voidbuttons()
        local x, rows

        for x = 1, T.VOID_PAGES do
            B.I.Frame.Create(T.F.mainframe.void, T.F.mainframe.void[x], x)
            rows = B.I.Grid.Create(T.F.mainframe.void[x], T.NUM_VOID_SLOTS)
        end

        T.F.mainframe.void:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.void:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.I.Void:Click(...)

        local frame = self:GetParent()

        if not T.ISVOIDOPEN then
            frame.character:Hide()
        elseif T.ISVOIDOPEN and frame.character:IsVisible() then
            -- move bags
            B.AnchorBags(frame.void)
        end

        frame.bank:Hide()
        frame.reagent:Hide()
        frame.guild:Hide()
        frame.search:Hide()

        if frame.void:IsVisible() then
            frame.lastbank = nil
            frame.void:Hide()
        else
            frame.lastbank = "void"
            frame.void:Show()
        end
    end

    function B.I.Void:Show(...)
        T.F.mainframe.voidbutton:LockHighlight()
    end

    function B.I.Void:Hide()
        T.F.mainframe.voidbutton:UnlockHighlight()
    end

    function B.I.Void.Tabs:Click(...)
        local parent = self:GetParent()

        parent[parent.lasttab].frame:Hide()
        parent[parent.lasttab].tab:SetBackdropBorderColor(0, 0, 0, 1)

        self:SetBackdropBorderColor(1, 1, 1, 1)
        parent[self.index].frame:Show()
        parent.lasttab = self.index

        if T.ISVOIDOPEN then
            VoidStorage_SetPageNumber(self.index)
        end
    end

--[[//////////////////////////////////////////////////

    GUILD FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    local create_guildbuttons, create_guildtabs

    function create_guild()

        create_guildtabs()
        create_guildbuttons()
    end

    function create_guildtabs()
        local x, lasttab

        for x = 1, T.MAX_GUILDBANK_TABS do
            T.F.mainframe.guild[x] = {}
            T.F.mainframe.guild[x].tab = B.I.Tab.Create(T.F.mainframe.guild)
            T.F.mainframe.guild[x].tab.index = x
            T.F.mainframe.guild[x].tab:SetScript("OnClick", B.I.Guild.Tabs.Click)
            T.F.mainframe.guild[x].tab:SetScript("OnEnter", B.I.Guild.Tabs.Enter)
            T.F.mainframe.guild[x].tab:SetScript("OnLeave", B.I.Guild.Tabs.Leave)

            if x == 1 then
                T.F.mainframe.guild[x].tab:SetPoint("TOPLEFT", T.F.mainframe.guild, "TOPRIGHT")
                T.F.mainframe.guild[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
            else
                T.F.mainframe.guild[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = T.F.mainframe.guild[x].tab
        end
    end

    function create_guildbuttons()
        local x, rows

        for x = 1, T.MAX_GUILDBANK_TABS do
            B.I.Frame.Create(T.F.mainframe.guild, T.F.mainframe.guild[x], x)
            rows = B.I.Grid.Create(T.F.mainframe.guild[x], T.MAX_GUILDBANK_SLOTS_PER_TAB, nil, nil, B.I.Guild.SplitStack)
        end

        T.F.mainframe.guild:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.guild:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.I.Guild:Click(...)

        local frame = self:GetParent()

        if not T.ISGUILDOPEN then
            frame.character:Hide()
        elseif T.ISGUILDOPEN and frame.character:IsVisible() then
            -- move bags
            B.AnchorBags(frame.guild)
        end

        frame.bank:Hide()
        frame.reagent:Hide()
        frame.void:Hide()
        frame.search:Hide()

        if frame.guild:IsVisible() then
            frame.lastbank = nil
            frame.guild:Hide()
        else
            frame.lastbank = "guild"
            frame.guild:Show()
        end
    end

    function B.I.Guild:Enter()
        local name = B.GetPlayerBank('guild')['name']

        if name then
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
            GameTooltip:SetText(name)
            GameTooltip:Show()
        end
    end

    function B.I.Guild:Leave()
        GameTooltip:Hide()
    end

    function B.I.Guild:Show()
        T.F.mainframe.guildbutton:LockHighlight()
        T.F.mainframe.guild[T.F.mainframe.guild.lasttab].tab:Click()
    end

    function B.I.Guild:Hide()
        T.F.mainframe.guildbutton:UnlockHighlight()
    end

    function B.I.Guild.Tabs:Click(...)
        local parent = self:GetParent()

        parent[parent.lasttab].frame:Hide()
        parent[parent.lasttab].tab:SetBackdropBorderColor(0, 0, 0, 1)

        self:SetBackdropBorderColor(1, 1, 1, 1)
        parent[self.index].frame:Show()
        parent.lasttab = self.index

        if T.ISGUILDOPEN then
            _G['GuildBankTab'..self.index..'Button']:Click()
        end
    end

    function B.I.Guild.Tabs:Enter()
        if T.F.mainframe.layout > 2 then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT", 10, -10)
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        end
        if self.name then GameTooltip:SetText(self.name) end
        GameTooltip:Show()
    end

    function B.I.Guild.Tabs:Leave()
        GameTooltip:Hide()
    end

    function B.I.Guild.SplitStack(button, split)
        SplitGuildBankItem(T.F.mainframe.guild.lasttab, button:GetID(), split)
    end

--[[//////////////////////////////////////////////////

    SEARCH FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    local create_searcheditbox, create_countbutton, create_searchbutton, create_searchscroll

    function create_search()

        create_searcheditbox()
        create_countbutton()
        create_searchscroll()
    end

    function create_searcheditbox()

        local editbox = CreateFrame("EditBox", "BW_Searchbar", T.F.mainframe, "BagSearchBoxTemplate")
        editbox:SetHeight(25)
        editbox:SetWidth(192)
        editbox:SetAutoFocus(false)
        editbox:SetPoint("LEFT", T.F.mainframe.guildbutton, "RIGHT", 10, 0)
        editbox:SetHyperlinksEnabled(true)
        editbox:SetMultiLine(false)
        editbox:SetMaxLetters(75)

        local fontName, fontHeight = editbox:GetFont()
        editbox:SetFont(fontName, fontHeight + 2)

        editbox:HookScript("OnEscapePressed", B.I.Search.EscapePressed)
        editbox:HookScript("OnTextChanged", B.I.Search.TextChanged)

        B.I.RemoveEdge(editbox)
        tinsert(ITEM_SEARCHBAR_LIST, "BW_Searchbar")
        T.F.mainframe.searchbar = editbox
    end

    function create_countbutton()

        local button = B.I.Main.Frame.Button("|cffffffff|r")
        button:SetWidth(64)
        button:SetPoint("LEFT", T.F.mainframe.searchbar, "RIGHT")
        button:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        button.count = 0
        button:Hide()

        button:SetScript("OnHide", B.I.Search.Count.Hide)
        button:SetScript("OnClick", B.I.Search.Count.Click)

        button.labelcount = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.labelcount:SetFont(T.CONFIG['FontQuantity'] == "default" and T.FONTS[2] or not T.CONFIG['FontQuantity'] and T.FONTS[2] or T.FONTS['quantity'], 14) -- Resize string
        button.labelcount:SetTextColor(0, 0, 0)
        button.labelcount:SetJustifyH("CENTER")
        button.labelcount:SetPoint("CENTER")

        T.F.mainframe.countbutton = button
    end

    function B.I.Search.Count:Hide()
        T.F.mainframe.search:Hide()
    end

    function B.I.Search.Count:Click(...)
        local frame = self:GetParent()

        frame.bank:Hide()
        frame.reagent:Hide()
        frame.character:Hide()
        frame.void:Hide()
        frame.guild:Hide()

        if frame.search:IsVisible() then
            frame.search:Hide()
            if frame.lastbank then frame[frame.lastbank]:Show() end
        else
            frame.search:Show()
        end
    end

    function create_searchscroll()

        T.F.mainframe.search.enddot = T.F.mainframe.search:CreateTexture(nil, "OVERLAY")
        T.F.mainframe.search.enddot:SetMask("Textures\\MinimapMask")
        T.F.mainframe.search.enddot:SetHeight(5)
        T.F.mainframe.search.enddot:SetWidth(5)
        T.F.mainframe.search.enddot:SetTexture("Interface\\Buttons\\WHITE8x8")
        T.F.mainframe.search.enddot:SetVertexColor(0.7, 0.7, 0.7, 1)
        T.F.mainframe.search.enddot:SetPoint("TOP", T.F.mainframe.search, "BOTTOM", 0, -5)

        T.F.mainframe.search:SetPoint("TOPLEFT", T.F.mainframe.searchbar, "BOTTOMLEFT", -10, 0)
        T.F.mainframe.search:SetPoint("TOPRIGHT", T.F.mainframe.countbutton, "BOTTOMRIGHT")

        create_scroll(T.F.mainframe.search, "FauxScrollFrameBW", B.ScrollbarUpdate, "BWScrollFrame", create_searchbutton, 32)
    end

    function create_searchbutton(index)

        local button = B.I.List.Button(index, T.F.mainframe.search.list)

        button:SetScript("OnEnter", B.I.Search.Button.Enter)
        button:SetScript("OnLeave", B.I.Search.Button.Leave)
        button:SetScript("OnClick", B.I.Search.Button.Click)

        local leftstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        leftstring:SetJustifyH("LEFT")
        leftstring:SetJustifyV("CENTER")
        leftstring:SetWordWrap(false)
        leftstring:SetPoint("LEFT", button.icon, "RIGHT", 10, 0)
        leftstring:SetPoint("RIGHT", button, "RIGHT", -T.F.mainframe.countbutton:GetWidth(), 0)
        button.leftstring = leftstring

       return button
    end

    function B.I.Search:TextChanged(userInput)

        local text = self:GetText()
        T.F.mainframe.countbutton.count = 0

        if text == "" or text == L[9007] then
            T.F.mainframe:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
            T.F.mainframe:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        end

        if text == L[9007] or text == "" then
            B.Search(false)
        else
            T.F.mainframe:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
            T.F.mainframe:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
            B.Search(text)
        end
    end

    function B.I.Search:EscapePressed()
        self:ClearFocus()
    end

    function B.I.Search.Button:Enter()
        if self.quest then
            self:SetBackdropBorderColor(1, 1, 0, 1)
        else
            local r, g, b = GetItemQualityColor(self.quality)
            self:SetBackdropBorderColor(r, g, b, 1)
        end
        B.I.Button.Enter(self, false)
    end

    function B.I.Search.Button:Leave()
        self:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        GameTooltip:Hide()
    end

    function B.I.Search.Button:Click(...)
        local location, page, player, faction, server

        location = self.location.loc
        if location ~= "alt" then
            page = tonumber(self.location.num)
        else
            server, player, faction = self.location.num:match"([^.]*).([^.]*).(.*)"
        end

        if location == "bank" then
            T.F.mainframe.bankbutton:Click()
        elseif location == "reagent" then
            T.F.mainframe.reagentbutton:Click()
        elseif location == "void" then
            T.F.mainframe.voidbutton:Click()
            T.F.mainframe.void[page].tab:Click()
        elseif location == "guild" then
            T.F.mainframe.guildbutton:Click()
            T.F.mainframe.guild[page].tab:Click()
        elseif location == "player" then
            T.F.mainframe.characterbutton:Click()
        elseif location == "alt" then
            B.load(server, player, faction == "H" and "Horde" or faction == "A" and "Alliance" or faction == "N" and "Neutral")
        elseif location == "mail" then
            T.F.mainframe.characterbutton:Click()
            if not T.F.mainframe.character.mail:IsVisible() then T.F.mainframe.character.mail.tab:Click() end
        elseif location == "auctions" then
            T.F.mainframe.characterbutton:Click()
            if not T.F.mainframe.character.auctions:IsVisible() then T.F.mainframe.character.auctions.tab:Click() end
        elseif location == "equip" then
            T.F.mainframe.characterbutton:Click()
            if not T.F.mainframe.character.equipment:IsVisible() then T.F.mainframe.character.tab:Click() end
        end
    end

--[[//////////////////////////////////////////////////

    CREATE COMMON FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    B.I.Button = {}
    function B.I.Button.Create(index, parent, splitstack)

        local button = CreateFrame("Button", nil, parent, "SecureActionButtonTemplate")

        button:SetID(index)
        button:SetHeight(T.BUTTON_HEIGHT)
        button:SetWidth(T.BUTTON_HEIGHT)
        button:SetFrameStrata("HIGH")

        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.2)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local icon = button:CreateTexture(nil, "ARTWORK", -1)

        icon:SetPoint("CENTER")
        icon:SetHeight(T.BUTTON_HEIGHT - 2)
        icon:SetWidth(T.BUTTON_HEIGHT - 2)
        icon:SetTexture("")
        icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        button.icon = icon

        local search = button:CreateTexture(nil, "OVERLAY")
        search:SetAllPoints()
        search:SetTexture("Interface\\Buttons\\WHITE8x8")
        search:SetVertexColor(0, 0, 0, 0.8)
        search:Hide()
        button.search = search

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")
        highlight:SetColorTexture(1, 1, 1, 0.2)
        highlight:SetAllPoints()
        highlight:SetBlendMode("ADD")
        button:SetHighlightTexture(highlight)

        button.text = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.text:SetTextColor(1, 1, 1)
        button.text:SetFont(T.FONTS['quantity'], math.floor((T.BUTTON_HEIGHT * 14)/32), T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.text:SetJustifyH("LEFT")
        button.text:SetJustifyV("CENTER")
        button.text:SetWordWrap(false)
        button.text:SetPoint("RIGHT", -3)
        button.text:SetPoint("BOTTOM", 0, 3)

        button.ilvl = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.ilvl:SetFont(T.FONTS['quantity'], math.floor((T.BUTTON_HEIGHT * 14)/32), T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.ilvl:SetJustifyH("LEFT")
        button.ilvl:SetJustifyV("CENTER")
        button.ilvl:SetWordWrap(false)
        button.ilvl:SetPoint("LEFT", 1)
        button.ilvl:SetPoint("BOTTOM", 0, 3)

        button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        button:SetAttribute("type", "macro")
        button:HookScript("PreClick", B.I.Button.PreClick)
        button:HookScript("OnClick", B.I.Button.Click)

        if splitstack then button.SplitStack = splitstack end

        return button
    end

    function B.I.Button:Update(elapsed)
        if not elapsed then return end
        if IsShiftKeyDown() and (IsModifiedClick("COMPAREITEMS") or GetCVarBool("alwaysCompareItems")) then
            GameTooltip_ShowCompareItem(GameTooltip)
            return
        end

        if GameTooltip.shoppingTooltips then -- Reset compares
            for _, frame in pairs(GameTooltip.shoppingTooltips) do
                frame:Hide()
            end
        end

        if IsControlKeyDown() then
            ShowInspectCursor()
        elseif T.ISMERCHANTOPEN and MerchantFrame.selectedTab == 1 and T.F.mainframe.lastbank == "character" and self.sell then
            SetCursor('BUY_CURSOR')
        else
            ResetCursor()
        end
    end

    function B.I.Button:PreClick(button)
        local BankFrameItem1 = BankFrameItem1
        local BankFrameTab1 = BankFrameTab1
        local BankFrameTab2 = BankFrameTab2

        if not IsShiftKeyDown() and button == "RightButton" and not T.F.mainframe.alt then
            local parent = self:GetParent()

            if self:GetParent() == T.F.mainframe.character.frame then -- special
                self:SetAttribute("macrotext", format("/use %d %d", B.GetBag(self)))
            elseif T.F.mainframe.lastbank == "guild" and T.ISGUILDOPEN then
                if GetCurrentGuildBankTab() ~= T.F.mainframe.guild.lasttab then T.F.mainframe.guild[T.F.mainframe.guild.lasttab].tab:Click() end
                AutoStoreGuildBankItem(T.F.mainframe.guild.lasttab, self:GetID())
            elseif T.F.mainframe.lastbank == "reagent" and T.ISBANKOPEN then
                if not ReagentBankFrame:IsVisible() then BankFrameTab2:Click() end
                UseContainerItem(T.REAGENTBANK_CONTAINER, self:GetID())
            elseif T.F.mainframe.lastbank == "void" and T.ISVOIDOPEN then
                if VoidStorageFrame.page ~= T.F.mainframe.void.lasttab then T.F.mainframe.void[T.F.mainframe.void.lasttab].tab:Click() end
                ClickVoidStorageSlot(T.F.mainframe.void.lasttab, self:GetID(), true)
            elseif T.F.mainframe.lastbank == "bank" and T.ISBANKOPEN then
                local bag, slot = B.GetBankBag(self)
                if not BankFrameItem1:IsVisible() then BankFrameTab1:Click() end
                if not IsBagOpen(bag) then OpenBag(bag) end
                UseContainerItem(B.GetBankBag(self))
            end
        end
    end

    function B.I.Button:Click(button)
        local BankFrameItem1 = BankFrameItem1
        local BankFrameTab1 = BankFrameTab1
        local BankFrameTab2 = BankFrameTab2

        if IsControlKeyDown() and button == "LeftButton" then
            DressUpItemLink(self.link)

        elseif IsShiftKeyDown() and button == "LeftButton" then

            if T.F.mainframe.searchbar:HasFocus() then
                T.F.mainframe.searchbar:SetText(self.name)
            elseif not T.F.mainframe.alt and self:GetParent() == T.F.mainframe.character.frame and self.count and self.count > 1 then -- special
                OpenStackSplitFrame(self.count, self, "BOTTOMLEFT", "TOPLEFT")
            elseif not T.F.mainframe.alt and T.F.mainframe.lastbank == "guild" and T.ISGUILDOPEN and self.count and self.count > 1 then
                if GetCurrentGuildBankTab() ~= T.F.mainframe.guild.lasttab then T.F.mainframe.guild[T.F.mainframe.guild.lasttab].tab:Click() end
                OpenStackSplitFrame(self.count, self, "BOTTOMLEFT", "TOPLEFT")
            elseif not T.F.mainframe.alt and T.F.mainframe.lastbank == "bank" and T.ISBANKOPEN and self.count and self.count > 1 then
                local bag, slot = B.GetBankBag(self)
                if not BankFrameItem1:IsVisible() then BankFrameTab1:Click() end
                if not IsBagOpen(bag) then OpenBag(bag) end
                OpenStackSplitFrame(self.count, self, "BOTTOMLEFT", "TOPLEFT")
            elseif not T.F.mainframe.alt and T.F.mainframe.lastbank == "reagent" and T.ISBANKOPEN and self.count and self.count > 1 then
                if not ReagentBankFrame:IsVisible() then BankFrameTab2:Click() end
                OpenStackSplitFrame(self.count, self, "BOTTOMLEFT", "TOPLEFT")
            else
                HandleModifiedItemClick(self.link)
            end

        elseif IsShiftKeyDown() and button == "RightButton" and not T.F.mainframe.alt then

            if ( IsModifiedClick("SOCKETITEM") ) then
                local parent = self:GetParent()
                if parent == T.F.mainframe.character.frame then
                    SocketContainerItem(B.GetBag(self))
                elseif parent == T.F.mainframe.character.equipment.frame then -- equipment tab
                    local id = self:GetID()
                    SocketInventoryItem(id == 18 and 19 or id)
                elseif T.F.mainframe.lastbank == "bank" then
                    SocketContainerItem(B.GetBankBag(self))
                end
            end


        elseif button == "LeftButton" and not T.F.mainframe.alt then

            if ( SpellCanTargetItem() or SpellCanTargetItemID() ) then
                -- Target the spell with the selected item
                if T.F.mainframe.lastbank == "character" then UseContainerItem(B.GetBag(self)); return end
            end
            
            if self:GetParent() == T.F.mainframe.character.frame then
                PickupContainerItem(B.GetBag(self))
            elseif T.F.mainframe.lastbank == "guild" and T.ISGUILDOPEN then
                if GetCurrentGuildBankTab() ~= T.F.mainframe.guild.lasttab then T.F.mainframe.guild[T.F.mainframe.guild.lasttab].tab:Click() end
                PickupGuildBankItem(T.F.mainframe.guild.lasttab, self:GetID())
            elseif T.F.mainframe.lastbank == "bank" and T.ISBANKOPEN then
                local bag, slot = B.GetBankBag(self)
                if not BankFrameItem1:IsVisible() then BankFrameTab1:Click() end
                if not IsBagOpen(bag) then OpenBag(bag) end
                PickupContainerItem(bag, slot)
            elseif T.F.mainframe.lastbank == "void" and T.ISVOIDOPEN then
                if VoidStorageFrame.page ~= T.F.mainframe.void.lasttab then T.F.mainframe.void[T.F.mainframe.void.lasttab].tab:Click() end
                ClickVoidStorageSlot(T.F.mainframe.void.lasttab, self:GetID(), false)
            elseif T.F.mainframe.lastbank == "reagent" and T.ISBANKOPEN then
                if not ReagentBankFrame:IsVisible() then BankFrameTab2:Click() end
                PickupContainerItem(T.REAGENTBANK_CONTAINER, self:GetID())
            end

        elseif button == "RightButton" then

            self:SetAttribute("macrotext", nil)
        end
    end

    function B.I.Button:Enter(cursorUpdate)

        if self.link then
            if cursorUpdate == nil or cursorUpdate == true then self:SetScript("OnUpdate", B.I.Button.Update) end
            GameTooltip:SetOwner(self, "ANCHOR_NONE")
            GameTooltip:ClearAllPoints()
            GameTooltip:SetPoint("TOPRIGHT", self, "TOPLEFT", -10, -20)

            if strmatch(self.link, "|Hbattlepet:") then
                -- Battle pet link
                local _, speciesID, level, breedQuality, maxHealth, power, speed, name = strsplit(":", self.link)
                BattlePetToolTip_Show(tonumber(speciesID), tonumber(level), tonumber(breedQuality), tonumber(maxHealth), tonumber(power), tonumber(speed), name)
                local r, g, b = GetItemQualityColor(tonumber(breedQuality))
                BattlePetTooltip:SetBackdropBorderColor(r, g, b)
            else
                GameTooltip:SetHyperlink(self.link)
                if self.quest then
                    GameTooltip:SetBackdropBorderColor(1, 1, 0)
                elseif self.quality then
                    local r, g, b = GetItemQualityColor(self.quality)
                    GameTooltip:SetBackdropBorderColor(r, g, b)
                end

                GameTooltip:Show()
            end
        end
    end

    function B.I.Button:Leave(...)
        ResetCursor()
        self:SetScript("OnUpdate", nil)
        GameTooltip_Hide() -- Blizzard function to close both gametooltip and battle pets
    end

    B.I.List = {}
    function B.I.List.Button(index, parent)

        local button = CreateFrame("Button", nil, parent)

        button:SetHeight(32)
        button:SetWidth(parent:GetWidth())
        button:SetPoint("TOP")
        button:SetPoint("LEFT")
        button:SetPoint("RIGHT")
        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.7)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")

        highlight:SetHeight(button:GetHeight() - 2)
        highlight:SetWidth(button:GetWidth() - 2)
        highlight:SetPoint("CENTER")
        highlight:SetColorTexture(1, 1, 1, 0.1)
        highlight:SetBlendMode("ADD")
        button:SetHighlightTexture(highlight)

        local icon = button:CreateTexture(nil, "ARTWORK", -1)

        icon:SetHeight(28)
        icon:SetWidth(28)
        icon:SetTexture("")
        icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        icon:SetPoint("TOPLEFT", 2, -2)
        button.icon = icon

        button.text = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.text:SetTextColor(1, 1, 1)
        button.text:SetFont(T.FONTS['quantity'], 13, T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.text:SetJustifyH("LEFT")
        button.text:SetJustifyV("CENTER")
        button.text:SetWordWrap(false)
        button.text:SetPoint("RIGHT", icon)
        button.text:SetPoint("BOTTOM", icon, 0, 3)

        button.ilvl = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.ilvl:SetFont(T.FONTS['quantity'], 13, T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.ilvl:SetJustifyH("LEFT")
        button.ilvl:SetJustifyV("CENTER")
        button.ilvl:SetWordWrap(false)
        button.ilvl:SetPoint("LEFT", icon)
        button.ilvl:SetPoint("BOTTOM", icon, 0, 1)

        button.rightstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.rightstring:SetJustifyH("CENTER")
        button.rightstring:SetJustifyV("CENTER")
        button.rightstring:SetPoint("RIGHT", -10, 0)
        button.rightstring:SetWidth(45)
        button.rightstring:SetWordWrap(false)
        button.rightstring:SetTextColor(1, 1, 1)

        button.rightstring.bg = button:CreateTexture(nil, "ARTWORK")
        button.rightstring.bg:SetPoint("RIGHT", -2, 0)
        button.rightstring.bg:SetHeight(button:GetHeight() - 4)
        button.rightstring.bg:SetWidth(button.rightstring:GetWidth() + 15)
        button.rightstring.bg:SetTexture("Interface\\Buttons\\WHITE8x8")
        button.rightstring.bg:SetVertexColor(0.05, 0.05, 0.05, 1)

        button.leftstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.leftstring:SetJustifyH("LEFT")
        button.leftstring:SetJustifyV("CENTER")
        button.leftstring:SetWordWrap(false)
        button.leftstring:SetWidth(button:GetWidth() - button.rightstring.bg:GetWidth())

        button.altstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
        button.altstring:SetJustifyH("CENTER")
        button.altstring:SetJustifyV("TOP")
        button.altstring:SetWidth(button.rightstring.bg:GetWidth() - 4)
        button.altstring:SetPoint("RIGHT", -2, 5)
        button.altstring:SetWordWrap(false)
        button.altstring:SetTextColor(1, 1, 1)

        button.altstring.server = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
        button.altstring.server:SetJustifyH("CENTER")
        button.altstring.server:SetJustifyV("BOTTOM")
        button.altstring.server:SetWidth(button.rightstring.bg:GetWidth() - 4)
        button.altstring.server:SetPoint("RIGHT", -2, -5)
        button.altstring.server:SetFont(button.altstring.server:GetFont(), 9)
        button.altstring.server:SetWordWrap(false)
        button.altstring.server:SetTextColor(1, 1, 1)

        button.altstring.faction = button:CreateTexture(nil, "ARTWORK")
        button.altstring.faction:SetPoint("RIGHT", 2, 0)
        button.altstring.faction:SetHeight(button:GetHeight() - 4)
        button.altstring.faction:SetWidth(3)
        button.altstring.faction:SetTexture("Interface\\Buttons\\WHITE8x8")
        button.altstring.faction:SetVertexColor(0.02, 0.76, 0.31, 1)

        return button
    end

    B.I.TabList = {}
    function B.I.TabList.Button(index, parent)

        local button = CreateFrame("Button", nil, parent)

        button:SetHeight(28)
        button:SetWidth(parent:GetWidth())
        button:SetPoint("TOP")
        button:SetPoint("LEFT")
        button:SetPoint("RIGHT")
        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.7)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")

        highlight:SetHeight(button:GetHeight() - 2)
        highlight:SetWidth(button:GetWidth() - 2)
        highlight:SetPoint("CENTER")
        highlight:SetColorTexture(1, 1, 1, 0.1)
        highlight:SetBlendMode("ADD")
        button:SetHighlightTexture(highlight)

        local icon = button:CreateTexture(nil, "ARTWORK", -1)

        icon:SetHeight(24)
        icon:SetWidth(24)
        icon:SetTexture("")
        icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        icon:SetPoint("TOPLEFT", 2, -2)
        button.icon = icon

        button.text = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.text:SetTextColor(1, 1, 1)
        button.text:SetFont(T.FONTS['quantity'], 12, T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.text:SetJustifyH("LEFT")
        button.text:SetJustifyV("CENTER")
        button.text:SetWordWrap(false)
        button.text:SetPoint("RIGHT", icon)
        button.text:SetPoint("BOTTOM", icon, 0, 3)

        button.ilvl = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.ilvl:SetFont(T.FONTS['quantity'], 12, T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.ilvl:SetJustifyH("LEFT")
        button.ilvl:SetJustifyV("CENTER")
        button.ilvl:SetWordWrap(false)
        button.ilvl:SetPoint("LEFT", icon, 1)
        button.ilvl:SetPoint("BOTTOM", icon, 0, 1)

        local price = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
        price:SetTextColor(1, 1, 1)
        price:SetFont(price:GetFont(), 12)
        price:SetJustifyH("CENTER")
        price:SetJustifyV("CENTER")
        price:SetWordWrap(false)
        price:SetWidth(55)
        price:SetPoint("RIGHT", -10, 0)
        button.rightstring = price

        button.rightstring.bg = button:CreateTexture(nil, "ARTWORK")
        button.rightstring.bg:SetPoint("RIGHT", -2, 0)
        button.rightstring.bg:SetHeight(button:GetHeight() - 4)
        button.rightstring.bg:SetWidth(button.rightstring:GetWidth() + 15)
        button.rightstring.bg:SetTexture("Interface\\Buttons\\WHITE8x8")
        button.rightstring.bg:SetVertexColor(0.05, 0.05, 0.05, 1)

        button.rightstring.time = button:CreateTexture(nil, "ARTWORK")
        button.rightstring.time:SetPoint("RIGHT", -3, 0)
        button.rightstring.time:SetHeight(button:GetHeight())
        button.rightstring.time:SetWidth(3)
        button.rightstring.time:SetTexture("Interface\\Buttons\\WHITE8x8")
        button.rightstring.time:SetVertexColor(0.02, 0.76, 0.31, 1)

        button.leftstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.leftstring:SetJustifyH("LEFT")
        button.leftstring:SetJustifyV("CENTER")
        button.leftstring:SetWordWrap(false)
        button.leftstring:SetFont(price:GetFont(), 10)
        button.leftstring:SetWidth(button:GetWidth() - button.rightstring.bg:GetWidth())

        return button
    end

    B.I.Tab = {}
    function B.I.Tab.Create(parent, name, inherit)

        local button = CreateFrame("Button", name, parent, inherit)
        button:SetHeight(T.BUTTON_HEIGHT)
        button:SetWidth(T.BUTTON_HEIGHT)
        button:SetFrameStrata("HIGH")

        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.2)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local normal = button:CreateTexture(nil, "OVERLAY")
        normal:SetPoint("CENTER")
        normal:SetHeight(button:GetHeight() - 2)
        normal:SetWidth(button:GetWidth() - 2)
        normal:SetTexture("")
        normal:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        button:SetNormalTexture(normal)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")
        highlight:SetColorTexture(1, 1, 1, 0.2)
        highlight:SetAllPoints()
        highlight:SetBlendMode("ADD")
        button:SetHighlightTexture(highlight)

        local search = button:CreateTexture(nil, "OVERLAY")

        search:SetAllPoints(normal)
        search:SetTexture("Interface\\Buttons\\WHITE8x8")
        search:SetVertexColor(0, 0, 0, 0.8)
        search:Hide()
        button.search = search

        return button
    end

    B.I.Frame = {}
    function B.I.Frame.Create(parent, pframe, index)
        -- Setup base for holding our buttons
        pframe.buttons = {}

        local frame = CreateFrame("Frame", nil, parent)
        frame:SetPoint("BOTTOM")
        frame:SetPoint("TOPLEFT", 10, -20)
        frame:SetFrameStrata("HIGH")

        if index then frame:SetID(index) end
        if index and index > 1 then frame:Hide() end

        pframe.frame = frame
    end

    B.I.Grid = {}
    function B.I.Grid.Create(parent, maxslots, rowcounter, numcounter, splitstack)
        local x, button, lastbutton, lastrow, rows, num

        rows = rowcounter or 0
        num = numcounter or 0

        for x = 1, maxslots do

            button = B.I.Button.Create(x, parent.frame, splitstack)
            button:SetScript("OnEnter", B.I.Button.Enter)
            button:SetScript("OnLeave", B.I.Button.Leave)
            
            if x == 1 then -- First slot
                button:SetPoint("TOPLEFT")
                rows, num = 1, 1
                lastrow = button

            elseif num == T.ROW then -- Last column of the row
                button:SetPoint("TOP", lastrow, "BOTTOM", 0, -T.FRAMEWHITESPACE)
                rows, num = rows + 1, 1
                lastrow = button
            else
                button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", T.FRAMEWHITESPACE, 0)
                num = num + 1
            end
            parent.buttons[x] = button
            lastbutton = button
        end

        return rows, num
    end

    function B.I.Grid.Add(parent, maxslots, rows, num, splitstack)
        local x, button, lastbutton, lastrow, numslots

        numslots = (T.ROW * (rows - 1)) + num
        lastbutton = parent.buttons[numslots]
        lastrow = parent.buttons[T.ROW * (rows - 1) + 1]

        for x = numslots + 1, maxslots do

            if parent.buttons[x] then
                parent.buttons[x]:Show()

                if num == T.ROW then
                    rows, num = rows + 1, 1
                    lastrow = parent.buttons[x]
                else
                    num = num + 1
                end
                lastbutton = parent.buttons[x]
            else
                button = B.I.Button.Create(x, parent.frame, splitstack)
                button:SetScript("OnEnter", B.I.Button.Enter)
                button:SetScript("OnLeave", B.I.Button.Leave)

                if num == T.ROW then -- Last column of the row
                    button:SetPoint("TOP", lastrow, "BOTTOM", 0, -T.FRAMEWHITESPACE)
                    lastrow = button
                    rows, num = rows + 1, 1
                else
                    button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", T.FRAMEWHITESPACE, 0)
                    num = num + 1
                end
                parent.buttons[x] = button
                lastbutton = button
            end
        end

        return rows, num
    end

    function B.I.Grid.Remove(parent, maxslots, rows, num)
        local x, numslots, difference

        numslots = (T.ROW * (rows - 1)) + num
        difference = numslots - maxslots

        for x = numslots, (numslots - difference + 1), -1 do

            parent.buttons[x]:Hide()

            if num == 1 then
                rows, num = rows - 1, T.ROW
            else
                num = num - 1
            end
        end
        return rows, num
    end

    function B.I.Backdrop(frame)

        frame:SetBackdrop({ bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
                            edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
                            tile = false, tileSize = 32, edgeSize = 1,
                            insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        frame:SetBackdropBorderColor(0, 0, 0, 1)
        frame:SetBackdropColor(0, 0, 0, 0.5)
        frame:EnableMouse(true)
    end

    B.I.GradientButton = {}
    function B.I.GradientButton.Create(button, orientation)

        button:SetNormalTexture("")
        button:SetHighlightTexture("")
        button:SetPushedTexture("")
        button:SetDisabledTexture("")

        B.I.RemoveEdge(button)

        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8",
                            edgeFile = "Interface\\Buttons\\WHITE8x8",
                            edgeSize = 1,
                            insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        button:SetBackdropColor(0, 0, 0, 0)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local gradient = button:CreateTexture(nil, "BACKGROUND")

        gradient:SetColorTexture(1, 1, 1)
        gradient:SetAllPoints()
        gradient:SetGradient(orientation, 0, 0, 0, 0.15, 0.15, 0.15)
    end

    function B.I.RemoveEdge(frame)

        local name = frame:GetName()
        local left = frame.Left or name and _G[name.."Left"] or nil
        local middle = frame.Middle or name and (_G[name.."Middle"] or _G[name.."Mid"]) or nil
        local right = frame.Right or name and _G[name.."Right"] or nil

        if left then left:Hide() end
        if middle then middle:Hide() end
        if right then right:Hide() end
    end

    function B.I.Input(editbox)

        editbox:SetTextColor(1, 1, 1)
        editbox:SetTextInsets(5, 0, 0, 0)

        B.I.RemoveEdge(editbox)
        
        editbox:SetBackdrop({   bgFile = AS_backdrop,
                                edgeFile = AS_backdrop,
                                edgeSize = 1,
                                insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        editbox:SetBackdropColor(0, 0, 0, 0.8)
        editbox:SetBackdropBorderColor(1, 1, 1, 0.1)
        editbox:SetFontObject("GameFontNormal")

        editbox:HookScript("OnEscapePressed", function(self) self:ClearFocus() end)
    end

    function B.I.ScrollFrame(parent, name, onUpdate, buttons, height)

        local scrollframe = B.I.Scroll.Create(parent, name, onUpdate, height)
        local i, currentrow, previousrow

        parent.buttons = {}

        for i = 1, B.RowsToFit(parent:GetParent(), height) do
            parent.buttons[i] = buttons(i, parent)
            currentrow = parent.buttons[i]
            if i == 1 then
                currentrow:SetPoint("TOP")
            else
                currentrow:SetPoint("TOP", previousrow, "BOTTOM", 0, -1)
            end
            currentrow:Show()
            previousrow = currentrow
        end

        return scrollframe
    end

    B.I.Scroll = {}
    function B.I.Scroll.Create(parent, name, onUpdate, height)

        --[[note the anchors: the area of the scrollframe is the scrollable area
            that intercepts mousewheel to scroll. it does not include the scrollbar,
            which is anchored off the right ]]

        local scroll = CreateFrame("ScrollFrame", name, parent, "FauxScrollFrameTemplate")
        scroll:SetPoint("TOPLEFT", parent:GetParent(), "TOPLEFT")
        scroll:SetPoint("BOTTOMRIGHT", parent:GetParent(), "BOTTOMRIGHT")
        scroll.onUpdate = onUpdate
        scroll.height = height

        scroll:SetScript("OnShow", onUpdate)
        scroll:SetScript("OnVerticalScroll", B.I.Scroll.VerticalScroll)
        scroll.ScrollBar:SetMinMaxValues(0, 100)
        scroll.ScrollBar:Hide()

        return scroll
    end

    function B.I.Scroll:VerticalScroll(offset)
        FauxScrollFrame_OnVerticalScroll(self, offset, self.height or 32, self.onUpdate)
    end

    B.I.ColorPicker = {}
    function B.I.ColorPicker.Create(r, g, b, a, changedCallback)
        ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = nil, nil, nil
        ColorPickerFrame:SetColorRGB(r, g, b);
        ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a;
        ColorPickerFrame.previousValues = {r, g, b, a};
        ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = changedCallback, changedCallback, changedCallback;
        ColorPickerFrame:Hide(); -- Need to run the OnShow handler.
        ColorPickerFrame:Show();
    end

--[[//////////////////////////////////////////////////

    ANCHOR FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    function B.I.AnchorBagsTopLeft()
        -- DEFAULT LAYOUT
        T.F.mainframe.layout = 1
        B.AnchorFrames("TOP", "BOTTOM", "LEFT", "RIGHT")
    end

    function B.I.AnchorBagsBottomLeft()

        T.F.mainframe.layout = 2
        B.AnchorFrames("BOTTOM", "TOP", "LEFT", "RIGHT")
    end

    function B.I.AnchorBagsTopRight()

        T.F.mainframe.layout = 3
        B.AnchorFrames("TOP", "BOTTOM", "RIGHT", "LEFT")
    end

    function B.I.AnchorBagsBottomRight()

        T.F.mainframe.layout = 4
        B.AnchorFrames("BOTTOM", "TOP", "RIGHT", "LEFT")
    end

    function B.I.Main.Frame.ButtonHighlight(button, orientation)
        local highlight = button:GetHighlightTexture()

        highlight:ClearAllPoints()
        highlight:SetPoint(orientation.."LEFT", button, orientation.."LEFT", 0, orientation == "TOP" and 1 or -1)
        highlight:SetPoint(orientation.."RIGHT", button, orientation.."RIGHT", 0, orientation == "TOP" and 1 or -1)
        highlight:SetHeight(2)
    end

    function B.I.Main.Frame.ButtonOrder(orientation)

        T.F.mainframe.optionbutton:ClearAllPoints()
        T.F.mainframe.bankbutton:ClearAllPoints()
        T.F.mainframe.reagentbutton:ClearAllPoints()
        T.F.mainframe.characterbutton:ClearAllPoints()
        T.F.mainframe.voidbutton:ClearAllPoints()
        T.F.mainframe.guildbutton:ClearAllPoints()
        T.F.mainframe.searchbar:ClearAllPoints()
        T.F.mainframe.countbutton:ClearAllPoints()

        if orientation == "LEFT" then
            B.I.Main.Frame.ButtonLeft("LEFT", "RIGHT")
        else
            B.I.Main.Frame.ButtonRight("RIGHT", "LEFT")
        end
    end

    function B.I.Main.Frame.ButtonLeft(startpoint, direction)

        T.F.mainframe.optionbutton:SetPoint(startpoint, T.F.mainframe, startpoint)
        T.F.mainframe.optionbutton:SetPoint("BOTTOM", T.F.mainframe, "BOTTOM")
        T.F.mainframe.bankbutton:SetPoint(startpoint, T.F.mainframe.optionbutton, direction)
        T.F.mainframe.reagentbutton:SetPoint(startpoint, T.F.mainframe.bankbutton, direction)
        T.F.mainframe.characterbutton:SetPoint(startpoint, T.F.mainframe.reagentbutton, direction)
        T.F.mainframe.voidbutton:SetPoint(startpoint, T.F.mainframe.characterbutton, direction)
        T.F.mainframe.guildbutton:SetPoint(startpoint, T.F.mainframe.voidbutton, direction)
        T.F.mainframe.searchbar:SetPoint(startpoint, T.F.mainframe.guildbutton, direction, 10, 0)
        T.F.mainframe.countbutton:SetPoint(startpoint, T.F.mainframe.searchbar, direction)
    end

    function B.I.Main.Frame.ButtonRight(startpoint, direction)

        T.F.mainframe.optionbutton:SetPoint(startpoint, T.F.mainframe, startpoint)
        T.F.mainframe.optionbutton:SetPoint("BOTTOM", T.F.mainframe, "BOTTOM")
        T.F.mainframe.guildbutton:SetPoint(startpoint, T.F.mainframe.optionbutton, direction)
        T.F.mainframe.voidbutton:SetPoint(startpoint, T.F.mainframe.guildbutton, direction)
        T.F.mainframe.characterbutton:SetPoint(startpoint, T.F.mainframe.voidbutton, direction)
        T.F.mainframe.reagentbutton:SetPoint(startpoint, T.F.mainframe.characterbutton, direction)
        T.F.mainframe.bankbutton:SetPoint(startpoint, T.F.mainframe.reagentbutton, direction)
        T.F.mainframe.searchbar:SetPoint(startpoint, T.F.mainframe.bankbutton, direction, -10, 0)
        T.F.mainframe.countbutton:SetPoint(startpoint, T.F.mainframe.searchbar, direction)
    end

    function B.AnchorFrames(startpoint, endpoint, from, to)

        B.I.Main.Frame.ButtonOrder(from)

        _G['BWdropDownMenu']:ClearAllPoints()
        _G['BWdropDownMenu'].point = startpoint..from
        _G['BWdropDownMenu'].relativePoint = endpoint..from

        -- bank
        B.AnchorBank(startpoint, endpoint, from, to)
        -- reagent
        B.AnchorReagent(startpoint, endpoint, from, to)
        -- character
        B.AnchorCharacter(startpoint, endpoint, from, to)
        B.AnchorCharacterEquipment(startpoint, endpoint, from, to)
        -- void
        B.AnchorVoid(startpoint, endpoint, from, to)
        -- guild
        B.AnchorGuild(startpoint, endpoint, from, to)
        -- search
        B.AnchorSearchList(startpoint, endpoint, from, to)
        -- auctions
        B.AnchorAuctionList(startpoint, endpoint, from, to)
        -- mail
        B.AnchorMailList(startpoint, endpoint, from, to)
    end

    function B.AnchorBank(startpoint, endpoint, from, to)
        local lasttab

        T.F.mainframe.bank:ClearAllPoints()
        T.F.mainframe.bank:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.bankbutton, endpoint)

        for x = 1, T.NUM_BANKBAGSLOTS do
            T.F.mainframe.bank.bags[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.bank.bags[x].tab:SetPoint(startpoint..to, T.F.mainframe.bank, endpoint..to)
            else
                T.F.mainframe.bank.bags[x].tab:SetPoint(startpoint..to, lasttab, startpoint..from, to == "RIGHT" and -2 or 2, 0)
            end
            lasttab = T.F.mainframe.bank.bags[x].tab
        end

        T.F.mainframe.bank.tooltiptoggle:ClearAllPoints()
        T.F.mainframe.bank.tooltiptoggle:SetPoint("TOP"..to, to == "RIGHT" and -10 or 10, 0)

        T.F.mainframe.bank.bagstoggle:ClearAllPoints()
        T.F.mainframe.bank.bagstoggle:SetPoint("TOP"..to, T.F.mainframe.bank.tooltiptoggle, "TOP"..from, to == "RIGHT" and -5 or 5, 0)
    end

    function B.AnchorCharacter(startpoint, endpoint, from, to)
        local lasttab

        T.F.mainframe.character:ClearAllPoints()
        T.F.mainframe.character:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.characterbutton, endpoint)

        T.F.mainframe.character.tab:ClearAllPoints()
        T.F.mainframe.character.tab:SetPoint("TOP"..from, T.F.mainframe.character, "TOP"..to)

        T.F.mainframe.character.auctions.tab:ClearAllPoints()
        T.F.mainframe.character.auctions.tab:SetPoint("TOP"..from, T.F.mainframe.character.tab, "BOTTOM"..from, 0, -2)

        T.F.mainframe.character.mail.tab:ClearAllPoints()
        T.F.mainframe.character.mail.tab:SetPoint("TOP"..from, T.F.mainframe.character.auctions.tab, "BOTTOM"..from, 0, -2)

        T.F.mainframe.character.artifact:ClearAllPoints()
        T.F.mainframe.character.artifact:SetPoint("TOP"..from, T.F.mainframe.character.mail.tab, "BOTTOM"..from, 0, -2)

        for x = 1, T.NUM_BAG_SLOTS do
            T.F.mainframe.character.bags[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.character.bags[x].tab:SetPoint(startpoint..to, T.F.mainframe.character, endpoint..to)
            else
                T.F.mainframe.character.bags[x].tab:SetPoint(startpoint..to, lasttab, startpoint..from, to == "RIGHT" and -2 or 2, 0)
            end
            lasttab = T.F.mainframe.character.bags[x].tab
        end

        T.F.mainframe.character.tooltiptoggle:ClearAllPoints()
        T.F.mainframe.character.tooltiptoggle:SetPoint("TOP"..to, to == "RIGHT" and -10 or 10, 0)

        T.F.mainframe.character.bagstoggle:ClearAllPoints()
        T.F.mainframe.character.bagstoggle:SetPoint("TOP"..to, T.F.mainframe.character.tooltiptoggle, "TOP"..from, to == "RIGHT" and -5 or 5, 0)
    end

    function B.AnchorCharacterEquipment(startpoint, endpoint, from, to)

        T.F.mainframe.character.equipment:ClearAllPoints()
        T.F.mainframe.character.equipment:SetPoint("TOP"..from, T.F.mainframe.character.tab, "TOP"..to)
    end

    function B.AnchorReagent(startpoint, endpoint, from, to)

        T.F.mainframe.reagent:ClearAllPoints()
        T.F.mainframe.reagent:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.reagentbutton, endpoint)
    end

    function B.AnchorVoid(startpoint, endpoint, from, to)
        local lasttab

        T.F.mainframe.void:ClearAllPoints()
        T.F.mainframe.void:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.voidbutton, endpoint)

        for x = 1, T.VOID_PAGES do
            T.F.mainframe.void[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.void[x].tab:SetPoint("TOP"..from, T.F.mainframe.void, "TOP"..to)
            else
                T.F.mainframe.void[x].tab:SetPoint("TOP"..from, lasttab, "BOTTOM"..from, 0, -4)
            end
            lasttab = T.F.mainframe.void[x].tab
        end
    end

    function B.AnchorGuild(startpoint, endpoint, from, to)
        local lasttab

        T.F.mainframe.guild:ClearAllPoints()
        T.F.mainframe.guild:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.guildbutton, endpoint)

        for x = 1, T.MAX_GUILDBANK_TABS do
            T.F.mainframe.guild[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.guild[x].tab:SetPoint("TOP"..from, T.F.mainframe.guild, "TOP"..to)
            else
                T.F.mainframe.guild[x].tab:SetPoint("TOP"..from, lasttab, "BOTTOM"..from, 0, -4)
            end
            lasttab = T.F.mainframe.guild[x].tab
        end
    end

    function B.AnchorSearchList(startpoint, endpoint, from, to)

        T.F.mainframe.search:ClearAllPoints()
        T.F.mainframe.search:SetPoint(startpoint..from, T.F.mainframe.searchbar, endpoint..from, from == "LEFT" and -10 or 10, 0)
        T.F.mainframe.search:SetPoint(startpoint..to, T.F.mainframe.countbutton, endpoint..to)

        T.F.mainframe.search.enddot:ClearAllPoints()
        T.F.mainframe.search.enddot:SetPoint(startpoint, T.F.mainframe.search, endpoint, 0, startpoint == "TOP" and -5 or 5)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.countbutton, endpoint)

        T.F.mainframe.search.list:ClearAllPoints()
        T.F.mainframe.search.list:SetPoint(startpoint..from, T.F.mainframe.search, 0, -2)
        T.F.mainframe.search.list:SetPoint(endpoint..to, T.F.mainframe.search)
    end

    function B.AnchorAuctionList(startpoint, endpoint, from, to)

        T.F.mainframe.character.auctions:ClearAllPoints()
        T.F.mainframe.character.auctions:SetPoint("TOP"..from, T.F.mainframe.character.tab, "TOP"..to)

        T.F.mainframe.character.auctions.list:ClearAllPoints()
        T.F.mainframe.character.auctions.list:SetPoint("TOP"..from, T.F.mainframe.character.auctions)
        T.F.mainframe.character.auctions.list:SetPoint("BOTTOM"..to, T.F.mainframe.character.auctions)

        local x
        for x = 1, #T.F.mainframe.character.auctions.list.buttons do
            T.F.mainframe.character.auctions.list.buttons[x].rightstring.time:ClearAllPoints()
            T.F.mainframe.character.auctions.list.buttons[x].rightstring.time:SetPoint(to, to == "RIGHT" and 3 or -3, 0)
        end
    end

    function B.AnchorMailList(startpoint, endpoint, from, to)

        T.F.mainframe.character.mail:ClearAllPoints()
        T.F.mainframe.character.mail:SetPoint("TOP"..from, T.F.mainframe.character.tab, "TOP"..to)

        T.F.mainframe.character.mail.list:ClearAllPoints()
        T.F.mainframe.character.mail.list:SetPoint("TOP"..from, T.F.mainframe.character.mail)
        T.F.mainframe.character.mail.list:SetPoint("BOTTOM"..to, T.F.mainframe.character.mail)

        local x
        for x = 1, #T.F.mainframe.character.mail.list.buttons do
            T.F.mainframe.character.mail.list.buttons[x].rightstring.time:ClearAllPoints()
            T.F.mainframe.character.mail.list.buttons[x].rightstring.time:SetPoint(to, to == "RIGHT" and 2 or -2, 0)
        end
    end

    function B.AnchorBags(parent)

        T.F.mainframe.character:ClearAllPoints()
        if T.F.mainframe.layout < 3 then -- left side
            T.F.mainframe.character:SetPoint("TOPLEFT", parent, "TOPRIGHT", 36, 0)
        else -- right side
            T.F.mainframe.character:SetPoint("TOPRIGHT", parent, "TOPLEFT", -36, 0)
        end
    end

    local function resetbg_button(button)

        button:SetBackdropColor(T.TRANSPARENCY_SLOT.r, T.TRANSPARENCY_SLOT.g, T.TRANSPARENCY_SLOT.b, T.TRANSPARENCY_SLOT.a)
        if not button.name or button.quality < 2 then
            button:SetBackdropBorderColor(T.BORDER_SLOT.r, T.BORDER_SLOT.g, T.BORDER_SLOT.b, T.BORDER_SLOT.a)
        end
    end

    function B.AdjustTransparency()
        local buttons

        T.F.mainframe.bank:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.bank:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.reagent:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.reagent:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.character:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.character:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.character.equipment:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.character.equipment:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.void:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.void:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.guild:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.guild:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)

        T.F.mainframe.bank.tooltiptoggle:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe.bank.tooltiptoggle:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.bank.bagstoggle:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe.bank.bagstoggle:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.character.tooltiptoggle:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe.character.tooltiptoggle:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.character.bagstoggle:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe.character.bagstoggle:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe.countbutton:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)

        for x = 1, #T.F.mainframe.bank.buttons do
            resetbg_button(T.F.mainframe.bank.buttons[x])
        end

        for b = 1, T.REAGENT_PAGES do
            buttons = T.F.mainframe.reagent[b].buttons
            for x = 1, #buttons do
                resetbg_button(buttons[x])
            end
        end

        for x = 1, #T.F.mainframe.character.buttons do
            resetbg_button(T.F.mainframe.character.buttons[x])
        end

        for x = 1, #T.F.mainframe.character.equipment.buttons do
            resetbg_button(T.F.mainframe.character.equipment.buttons[x])
        end

        for b = 1, T.VOID_PAGES do
            buttons = T.F.mainframe.void[b].buttons
            for x = 1, #buttons do
                resetbg_button(buttons[x])
            end
        end

        for b = 1, T.MAX_GUILDBANK_TABS do
            buttons = T.F.mainframe.guild[b].buttons
            for x = 1, #buttons do
                resetbg_button(buttons[x])
            end
        end
    end

    local function resetfont(button)
        local _, size = button:GetFont()
        button:SetFont(T.FONTS['quantity'], size, T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
    end

    local function resetfont_button(button)

        resetfont(button.text)
        resetfont(button.ilvl)
    end

    function B.ResetFontButtons()
        local b, x, buttons

        for x = 1, #T.F.mainframe.bank.buttons do
            resetfont_button(T.F.mainframe.bank.buttons[x])
        end

        for b = 1, T.REAGENT_PAGES do
            buttons = T.F.mainframe.reagent[b].buttons
            for x = 1, #buttons do
                resetfont_button(buttons[x])
            end
        end

        for x = 1, #T.F.mainframe.character.buttons do
            resetfont_button(T.F.mainframe.character.buttons[x])
        end

        for x = 1, #T.F.mainframe.character.equipment.buttons do
            resetfont_button(T.F.mainframe.character.equipment.buttons[x])
        end

        for b = 1, T.VOID_PAGES do
            buttons = T.F.mainframe.void[b].buttons
            for x = 1, #buttons do
                resetfont_button(buttons[x])
            end
        end

        for b = 1, T.MAX_GUILDBANK_TABS do
            buttons = T.F.mainframe.guild[b].buttons
            for x = 1, #buttons do
                resetfont_button(buttons[x])
            end
        end

        for x = 1, #T.F.mainframe.search.list.buttons do
            resetfont_button(T.F.mainframe.search.list.buttons[x])
        end

        for x = 1, #T.F.mainframe.character.auctions.list.buttons do
            resetfont_button(T.F.mainframe.character.auctions.list.buttons[x])
        end

        for x = 1, #T.F.mainframe.character.mail.list.buttons do
            resetfont_button(T.F.mainframe.character.mail.list.buttons[x])
        end

        local _, size = T.F.mainframe.countbutton.labelcount:GetFont()
        T.F.mainframe.countbutton.labelcount:SetFont(T.CONFIG['FontQuantity'] == "default" and T.FONTS[2] or not T.CONFIG['FontQuantity'] and T.FONTS[2] or T.FONTS['quantity'], size)
    end

